@@ -7,7 +7,6 @@ import (
 	"fmt"
 	"math/rand"
 	"net/http"
-	"runtime/debug"
 	"sort"
 	"strings"
 	"sync"
@@ -175,14 +174,9 @@ func parseFile(args parseArgs) {
 	defer func() {
 		r := recover()
 		if r != nil {
-			stack := strings.TrimSpace(string(debug.Stack()))
-			tracker := logger.MakeLineColumnTracker(&source)
-			data := logger.RangeData(&tracker, logger.Range{}, fmt.Sprintf("panic: %v", r))
-			data.Location.LineText = fmt.Sprintf("%s\n%s", data.Location.LineText, stack)
-			args.log.AddMsg(logger.Msg{
-				Kind: logger.Error,
-				Data: data,
-			})
+			args.log.AddWithNotes(logger.Error, nil, logger.Range{},
+				fmt.Sprintf("panic: %v (while parsing %q)", r, source.PrettyPath),
+				[]logger.MsgData{{Text: helpers.PrettyPrintedStack()}})
 			args.results <- result
 		}
 	}()
@@ -190,12 +184,18 @@ func parseFile(args parseArgs) {
 	switch loader {
 	case config.LoaderJS:
 		ast, ok := args.caches.JSCache.Parse(args.log, source, js_parser.OptionsFromConfig(&args.options))
+		if len(ast.Parts) <= 1 { // Ignore the implicitly-generated namespace export part
+			result.file.inputFile.SideEffects.Kind = graph.NoSideEffects_EmptyAST
+		}
 		result.file.inputFile.Repr = &graph.JSRepr{AST: ast}
 		result.ok = ok
 
 	case config.LoaderJSX:
 		args.options.JSX.Parse = true
 		ast, ok := args.caches.JSCache.Parse(args.log, source, js_parser.OptionsFromConfig(&args.options))
+		if len(ast.Parts) <= 1 { // Ignore the implicitly-generated namespace export part
+			result.file.inputFile.SideEffects.Kind = graph.NoSideEffects_EmptyAST
+		}
 		result.file.inputFile.Repr = &graph.JSRepr{AST: ast}
 		result.ok = ok
 
@@ -203,6 +203,9 @@ func parseFile(args parseArgs) {
 		args.options.TS.Parse = true
 		args.options.TS.NoAmbiguousLessThan = loader == config.LoaderTSNoAmbiguousLessThan
 		ast, ok := args.caches.JSCache.Parse(args.log, source, js_parser.OptionsFromConfig(&args.options))
+		if len(ast.Parts) <= 1 { // Ignore the implicitly-generated namespace export part
+			result.file.inputFile.SideEffects.Kind = graph.NoSideEffects_EmptyAST
+		}
 		result.file.inputFile.Repr = &graph.JSRepr{AST: ast}
 		result.ok = ok
 
@@ -210,6 +213,9 @@ func parseFile(args parseArgs) {
 		args.options.TS.Parse = true
 		args.options.JSX.Parse = true
 		ast, ok := args.caches.JSCache.Parse(args.log, source, js_parser.OptionsFromConfig(&args.options))
+		if len(ast.Parts) <= 1 { // Ignore the implicitly-generated namespace export part
+			result.file.inputFile.SideEffects.Kind = graph.NoSideEffects_EmptyAST
+		}
 		result.file.inputFile.Repr = &graph.JSRepr{AST: ast}
 		result.ok = ok
 
@@ -317,7 +323,7 @@ func parseFile(args parseArgs) {
 			message = fmt.Sprintf("Do not know how to load path: %s", source.PrettyPath)
 		}
 		tracker := logger.MakeLineColumnTracker(args.importSource)
-		args.log.AddRangeError(&tracker, args.importPathRange, message)
+		args.log.Add(logger.Error, &tracker, args.importPathRange, message)
 	}
 
 	// This must come before we send on the "results" channel to avoid deadlock
@@ -406,8 +412,11 @@ func parseFile(args parseArgs) {
 				// All "require.resolve()" imports should be external because we don't
 				// want to waste effort traversing into them
 				if record.Kind == ast.ImportRequireResolve {
-					if !record.HandlesImportErrors && (resolveResult == nil || !resolveResult.IsExternal) {
-						args.log.AddRangeWarning(&tracker, record.Range,
+					if resolveResult != nil && resolveResult.IsExternal {
+						// Allow path substitution as long as the result is external
+						result.resolveResults[importRecordIndex] = resolveResult
+					} else if !record.HandlesImportErrors {
+						args.log.Add(logger.Warning, &tracker, record.Range,
 							fmt.Sprintf("%q should be marked as external for use with \"require.resolve\"", record.Path.Text))
 					}
 					continue
@@ -421,37 +430,47 @@ func parseFile(args parseArgs) {
 					if !didLogError && !record.HandlesImportErrors {
 						hint := ""
 						if resolver.IsPackagePath(record.Path.Text) {
+							hint = fmt.Sprintf("You can mark the path %q as external to exclude it from the bundle, which will remove this error.", record.Path.Text)
 							if record.Kind == ast.ImportRequire {
-								hint = ", or surround it with try/catch to handle the failure at run-time"
+								hint += " You can also surround this \"require\" call with a try/catch block to handle this failure at run-time instead of bundle-time."
 							} else if record.Kind == ast.ImportDynamic {
-								hint = ", or add \".catch()\" to handle the failure at run-time"
+								hint += " You can also add \".catch()\" here to handle this failure at run-time instead of bundle-time."
 							}
-							hint = fmt.Sprintf(" (mark it as external to exclude it from the bundle%s)", hint)
 							if pluginName == "" && !args.fs.IsAbs(record.Path.Text) {
 								if query := args.res.ProbeResolvePackageAsRelative(absResolveDir, record.Path.Text, record.Kind); query != nil {
-									hint = fmt.Sprintf(" (use %q to reference the file %q)", "./"+record.Path.Text, args.res.PrettyPath(query.PathPair.Primary))
+									hint = fmt.Sprintf("Use the relative path %q to reference the file %q. "+
+										"Without the leading \"./\", the path %q is being interpreted as a package path instead.",
+										"./"+record.Path.Text, args.res.PrettyPath(query.PathPair.Primary), record.Path.Text)
 								}
 							}
 						}
 						if args.options.Platform != config.PlatformNode {
 							if _, ok := resolver.BuiltInNodeModules[record.Path.Text]; ok {
+								var how string
 								switch logger.API {
 								case logger.CLIAPI:
-									hint = " (use \"--platform=node\" when building for node)"
+									how = "--platform=node"
 								case logger.JSAPI:
-									hint = " (use \"platform: 'node'\" when building for node)"
+									how = "platform: 'node'"
 								case logger.GoAPI:
-									hint = " (use \"Platform: api.PlatformNode\" when building for node)"
+									how = "Platform: api.PlatformNode"
 								}
+								hint = fmt.Sprintf("The package %q wasn't found on the file system but is built into node. "+
+									"Are you trying to bundle for node? You can use %q to do that, which will remove this error.", record.Path.Text, how)
 							}
 						}
 						if absResolveDir == "" && pluginName != "" {
-							hint = fmt.Sprintf(" (the plugin %q didn't set a resolve directory)", pluginName)
+							hint = fmt.Sprintf("The plugin %q didn't set a resolve directory for the file %q, "+
+								"so esbuild did not search for %q on the file system.", pluginName, source.PrettyPath, record.Path.Text)
 						}
-						debug.LogErrorMsg(args.log, &source, record.Range, fmt.Sprintf("Could not resolve %q%s", record.Path.Text, hint))
+						var notes []logger.MsgData
+						if hint != "" {
+							notes = append(notes, logger.MsgData{Text: hint})
+						}
+						debug.LogErrorMsg(args.log, &source, record.Range, fmt.Sprintf("Could not resolve %q", record.Path.Text), notes)
 					} else if args.log.Level <= logger.LevelDebug && !didLogError && record.HandlesImportErrors {
-						args.log.AddRangeDebug(&tracker, record.Range,
-							fmt.Sprintf("Importing %q was allowed even though it could not be resolved because dynamic import failures appear to be handled here",
+						args.log.Add(logger.Debug, &tracker, record.Range,
+							fmt.Sprintf("Importing %q was allowed even though it could not be resolved because dynamic import failures appear to be handled here:",
 								record.Path.Text))
 					}
 					continue
@@ -550,7 +569,7 @@ func extractSourceMapFromComment(
 		if contents, err := parsed.DecodeData(); err == nil {
 			return logger.Path{Text: source.PrettyPath, IgnoredSuffix: "#sourceMappingURL"}, &contents
 		} else {
-			log.AddRangeWarning(&tracker, comment.Range, fmt.Sprintf("Unsupported source map comment: %s", err.Error()))
+			log.Add(logger.Warning, &tracker, comment.Range, fmt.Sprintf("Unsupported source map comment: %s", err.Error()))
 			return logger.Path{}, nil
 		}
 	}
@@ -561,14 +580,14 @@ func extractSourceMapFromComment(
 		path := logger.Path{Text: absPath, Namespace: "file"}
 		contents, err, originalError := fsCache.ReadFile(fs, absPath)
 		if log.Level <= logger.LevelDebug && originalError != nil {
-			log.AddRangeDebug(&tracker, comment.Range, fmt.Sprintf("Failed to read file %q: %s", res.PrettyPath(path), originalError.Error()))
+			log.Add(logger.Debug, &tracker, comment.Range, fmt.Sprintf("Failed to read file %q: %s", res.PrettyPath(path), originalError.Error()))
 		}
 		if err != nil {
 			if err == syscall.ENOENT {
 				// Don't report a warning because this is likely unactionable
 				return logger.Path{}, nil
 			}
-			log.AddRangeWarning(&tracker, comment.Range, fmt.Sprintf("Cannot read file %q: %s", res.PrettyPath(path), err.Error()))
+			log.Add(logger.Warning, &tracker, comment.Range, fmt.Sprintf("Cannot read file %q: %s", res.PrettyPath(path), err.Error()))
 			return logger.Path{}, nil
 		}
 		return path, &contents
@@ -578,7 +597,7 @@ func extractSourceMapFromComment(
 	return logger.Path{}, nil
 }
 
-func sanetizeLocation(res resolver.Resolver, loc *logger.MsgLocation) {
+func sanitizeLocation(res resolver.Resolver, loc *logger.MsgLocation) {
 	if loc != nil {
 		if loc.Namespace == "" {
 			loc.Namespace = "file"
@@ -612,17 +631,17 @@ func logPluginMessages(
 
 		// Sanitize the locations
 		for _, note := range msg.Notes {
-			sanetizeLocation(res, note.Location)
+			sanitizeLocation(res, note.Location)
 		}
 		if msg.Data.Location == nil {
-			msg.Data.Location = logger.LocationOrNil(&tracker, importPathRange)
+			msg.Data.Location = tracker.MsgLocationOrNil(importPathRange)
 		} else {
-			sanetizeLocation(res, msg.Data.Location)
+			sanitizeLocation(res, msg.Data.Location)
 			if msg.Data.Location.File == "" && importSource != nil {
 				msg.Data.Location.File = importSource.PrettyPath
 			}
 			if importSource != nil {
-				msg.Notes = append(msg.Notes, logger.RangeData(&tracker, importPathRange,
+				msg.Notes = append(msg.Notes, tracker.MsgData(importPathRange,
 					fmt.Sprintf("The plugin %q was triggered by this import", name)))
 			}
 		}
@@ -639,7 +658,7 @@ func logPluginMessages(
 			Kind:       logger.Error,
 			Data: logger.MsgData{
 				Text:       text,
-				Location:   logger.LocationOrNil(&tracker, importPathRange),
+				Location:   tracker.MsgLocationOrNil(importPathRange),
 				UserDetail: thrown,
 			},
 		})
@@ -728,10 +747,10 @@ func runOnResolvePlugins(
 			// Paths in the file namespace must be absolute paths
 			if result.Path.Namespace == "file" && !fs.IsAbs(result.Path.Text) {
 				if nsFromPlugin == "file" {
-					log.AddRangeError(&tracker, importPathRange,
+					log.Add(logger.Error, &tracker, importPathRange,
 						fmt.Sprintf("Plugin %q returned a path in the \"file\" namespace that is not an absolute path: %s", pluginName, result.Path.Text))
 				} else {
-					log.AddRangeError(&tracker, importPathRange,
+					log.Add(logger.Error, &tracker, importPathRange,
 						fmt.Sprintf("Plugin %q returned a non-absolute path: %s (set a namespace if this is not a file path)", pluginName, result.Path.Text))
 				}
 				return nil, true, resolver.DebugMeta{}
@@ -761,7 +780,7 @@ func runOnResolvePlugins(
 	// Warn when the case used for importing differs from the actual file name
 	if result != nil && result.DifferentCase != nil && !helpers.IsInsideNodeModules(absResolveDir) {
 		diffCase := *result.DifferentCase
-		log.AddRangeWarning(&tracker, importPathRange, fmt.Sprintf(
+		log.Add(logger.Warning, &tracker, importPathRange, fmt.Sprintf(
 			"Use %q instead of %q to avoid issues with case-sensitive file systems",
 			res.PrettyPath(logger.Path{Text: fs.Join(diffCase.Dir, diffCase.Actual), Namespace: "file"}),
 			res.PrettyPath(logger.Path{Text: fs.Join(diffCase.Dir, diffCase.Query), Namespace: "file"}),
@@ -868,14 +887,14 @@ func runOnLoadPlugins(
 			}, true
 		} else {
 			if log.Level <= logger.LevelDebug && originalError != nil {
-				log.AddDebug(nil, logger.Loc{}, fmt.Sprintf("Failed to read file %q: %s", source.KeyPath.Text, originalError.Error()))
+				log.Add(logger.Debug, nil, logger.Range{}, fmt.Sprintf("Failed to read file %q: %s", source.KeyPath.Text, originalError.Error()))
 			}
 			if err == syscall.ENOENT {
-				log.AddRangeError(&tracker, importPathRange,
+				log.Add(logger.Error, &tracker, importPathRange,
 					fmt.Sprintf("Could not read from file: %s", source.KeyPath.Text))
 				return loaderPluginResult{}, false
 			} else {
-				log.AddRangeError(&tracker, importPathRange,
+				log.Add(logger.Error, &tracker, importPathRange,
 					fmt.Sprintf("Cannot read file %q: %s", res.PrettyPath(source.KeyPath), err.Error()))
 				return loaderPluginResult{}, false
 			}
@@ -888,7 +907,7 @@ func runOnLoadPlugins(
 		if parsed, ok := resolver.ParseDataURL(source.KeyPath.Text); ok {
 			if mimeType := parsed.DecodeMIMEType(); mimeType != resolver.MIMETypeUnsupported {
 				if contents, err := parsed.DecodeData(); err != nil {
-					log.AddRangeError(&tracker, importPathRange,
+					log.Add(logger.Error, &tracker, importPathRange,
 						fmt.Sprintf("Could not load data URL: %s", err.Error()))
 					return loaderPluginResult{loader: config.LoaderNone}, true
 				} else {
@@ -1004,7 +1023,7 @@ func ScanBundle(
 	// Each bundling operation gets a separate unique key
 	uniqueKeyPrefix, err := generateUniqueKeyPrefix()
 	if err != nil {
-		log.AddError(nil, logger.Loc{}, fmt.Sprintf("Failed to read from randomness source: %s", err.Error()))
+		log.Add(logger.Error, nil, logger.Range{}, fmt.Sprintf("Failed to read from randomness source: %s", err.Error()))
 	}
 
 	s := scanner{
@@ -1099,8 +1118,8 @@ func (s *scanner) maybeParseFile(
 	if resolveResult.UseDefineForClassFieldsTS != config.Unspecified {
 		optionsClone.UseDefineForClassFields = resolveResult.UseDefineForClassFieldsTS
 	}
-	if resolveResult.PreserveUnusedImportsTS {
-		optionsClone.PreserveUnusedImportsTS = true
+	if resolveResult.UnusedImportsTS != config.UnusedImportsRemoveStmt {
+		optionsClone.UnusedImportsTS = resolveResult.UnusedImportsTS
 	}
 	optionsClone.TSTarget = resolveResult.TSTarget
 
@@ -1242,7 +1261,7 @@ func (s *scanner) preprocessInjectedFiles() {
 		absPathKey := canonicalFileSystemPathForWindows(absPath)
 
 		if duplicateInjectedFiles[absPathKey] {
-			s.log.AddError(nil, logger.Loc{}, fmt.Sprintf("Duplicate injected file %q", prettyPath))
+			s.log.Add(logger.Error, nil, logger.Range{}, fmt.Sprintf("Duplicate injected file %q", prettyPath))
 			continue
 		}
 
@@ -1250,7 +1269,7 @@ func (s *scanner) preprocessInjectedFiles() {
 		resolveResult := s.res.ResolveAbs(absPath)
 
 		if resolveResult == nil {
-			s.log.AddError(nil, logger.Loc{}, fmt.Sprintf("Could not resolve %q", prettyPath))
+			s.log.Add(logger.Error, nil, logger.Range{}, fmt.Sprintf("Could not resolve %q", prettyPath))
 			continue
 		}
 
@@ -1331,7 +1350,7 @@ func (s *scanner) addEntryPoints(entryPoints []EntryPoint) []graph.EntryPoint {
 				}
 			}
 		} else if s.log.Level <= logger.LevelDebug && originalError != nil {
-			s.log.AddDebug(nil, logger.Loc{}, fmt.Sprintf("Failed to read directory %q: %s", absPath, originalError.Error()))
+			s.log.Add(logger.Debug, nil, logger.Range{}, fmt.Sprintf("Failed to read directory %q: %s", absPath, originalError.Error()))
 		}
 	}
 
@@ -1365,20 +1384,28 @@ func (s *scanner) addEntryPoints(entryPoints []EntryPoint) []graph.EntryPoint {
 			)
 			if resolveResult != nil {
 				if resolveResult.IsExternal {
-					s.log.AddError(nil, logger.Loc{}, fmt.Sprintf("The entry point %q cannot be marked as external", entryPoint.InputPath))
+					s.log.Add(logger.Error, nil, logger.Range{}, fmt.Sprintf("The entry point %q cannot be marked as external", entryPoint.InputPath))
 				} else {
 					entryPointResolveResults[i] = resolveResult
 				}
 			} else if !didLogError {
-				hint := ""
+				var notes []logger.MsgData
 				if !s.fs.IsAbs(entryPoint.InputPath) {
 					if strings.ContainsRune(entryPoint.InputPath, '*') {
-						hint = " (glob syntax must be expanded first before passing the paths to esbuild)"
+						notes = append(notes, logger.MsgData{
+							Text: "It looks like you are trying to use glob syntax (i.e. \"*\") with esbuild. " +
+								"This syntax is typically handled by your shell, and isn't handled by esbuild itself. " +
+								"You must expand glob syntax first before passing your paths to esbuild.",
+						})
 					} else if query := s.res.ProbeResolvePackageAsRelative(entryPointAbsResolveDir, entryPoint.InputPath, ast.ImportEntryPoint); query != nil {
-						hint = fmt.Sprintf(" (use %q to reference the file %q)", "./"+entryPoint.InputPath, s.res.PrettyPath(query.PathPair.Primary))
+						notes = append(notes, logger.MsgData{
+							Text: fmt.Sprintf("Use the relative path %q to reference the file %q. "+
+								"Without the leading \"./\", the path %q is being interpreted as a package path instead.",
+								"./"+entryPoint.InputPath, s.res.PrettyPath(query.PathPair.Primary), entryPoint.InputPath),
+						})
 					}
 				}
-				debug.LogErrorMsg(s.log, nil, logger.Range{}, fmt.Sprintf("Could not resolve %q%s", entryPoint.InputPath, hint))
+				debug.LogErrorMsg(s.log, nil, logger.Range{}, fmt.Sprintf("Could not resolve %q", entryPoint.InputPath), notes)
 			}
 			entryPointWaitGroup.Done()
 		}(i, entryPoint)
@@ -1651,10 +1678,10 @@ func (s *scanner) processScannedFiles() []scannerFile {
 					// Using a JavaScript file with CSS "@import" is not allowed
 					otherFile := &s.results[record.SourceIndex.GetIndex()].file
 					if _, ok := otherFile.inputFile.Repr.(*graph.JSRepr); ok {
-						s.log.AddRangeError(&tracker, record.Range,
+						s.log.Add(logger.Error, &tracker, record.Range,
 							fmt.Sprintf("Cannot import %q into a CSS file", otherFile.inputFile.Source.PrettyPath))
 					} else if record.Kind == ast.ImportAtConditional {
-						s.log.AddRangeError(&tracker, record.Range,
+						s.log.Add(logger.Error, &tracker, record.Range,
 							"Bundling with conditional \"@import\" rules is not currently supported")
 					}
 
@@ -1663,12 +1690,12 @@ func (s *scanner) processScannedFiles() []scannerFile {
 					otherFile := &s.results[record.SourceIndex.GetIndex()].file
 					switch otherRepr := otherFile.inputFile.Repr.(type) {
 					case *graph.CSSRepr:
-						s.log.AddRangeError(&tracker, record.Range,
+						s.log.Add(logger.Error, &tracker, record.Range,
 							fmt.Sprintf("Cannot use %q as a URL", otherFile.inputFile.Source.PrettyPath))
 
 					case *graph.JSRepr:
 						if otherRepr.AST.URLForCSS == "" {
-							s.log.AddRangeError(&tracker, record.Range,
+							s.log.Add(logger.Error, &tracker, record.Range,
 								fmt.Sprintf("Cannot use %q as a URL", otherFile.inputFile.Source.PrettyPath))
 						}
 					}
@@ -1681,7 +1708,7 @@ func (s *scanner) processScannedFiles() []scannerFile {
 					otherFile := &s.results[record.SourceIndex.GetIndex()].file
 					if css, ok := otherFile.inputFile.Repr.(*graph.CSSRepr); ok {
 						if s.options.WriteToStdout {
-							s.log.AddRangeError(&tracker, record.Range,
+							s.log.Add(logger.Error, &tracker, record.Range,
 								fmt.Sprintf("Cannot import %q into a JavaScript file without an output path configured", otherFile.inputFile.Source.PrettyPath))
 						} else if !css.JSSourceIndex.IsValid() {
 							stubKey := otherFile.inputFile.Source.KeyPath
@@ -1730,7 +1757,12 @@ func (s *scanner) processScannedFiles() []scannerFile {
 						// Do not warn if this is from a plugin, since removing the import
 						// would cause the plugin to not run, and running a plugin is a side
 						// effect.
-						otherModule.SideEffects.Kind != graph.NoSideEffects_PureData_FromPlugin {
+						otherModule.SideEffects.Kind != graph.NoSideEffects_PureData_FromPlugin &&
+
+						// Do not warn if this has no side effects because the parsed AST
+						// is empty. This is the case for ".d.ts" files, for example.
+						otherModule.SideEffects.Kind != graph.NoSideEffects_EmptyAST {
+
 						var notes []logger.MsgData
 						var by string
 						if data := otherModule.SideEffects.Data; data != nil {
@@ -1744,10 +1776,10 @@ func (s *scanner) processScannedFiles() []scannerFile {
 									text = "\"sideEffects\" is false in the enclosing \"package.json\" file"
 								}
 								tracker := logger.MakeLineColumnTracker(data.Source)
-								notes = append(notes, logger.RangeData(&tracker, data.Range, text))
+								notes = append(notes, tracker.MsgData(data.Range, text))
 							}
 						}
-						s.log.AddRangeWarningWithNotes(&tracker, record.Range,
+						s.log.AddWithNotes(logger.Warning, &tracker, record.Range,
 							fmt.Sprintf("Ignoring this import because %q was marked as having no side effects%s",
 								otherModule.Source.PrettyPath, by), notes)
 					}
@@ -1780,21 +1812,22 @@ func (s *scanner) processScannedFiles() []scannerFile {
 
 			// Generate the input for the template
 			_, _, originalExt := logger.PlatformIndependentPathDirBaseExt(result.file.inputFile.Source.KeyPath.Text)
-			dir, base, ext := pathRelativeToOutbase(
+			dir, base := pathRelativeToOutbase(
 				&result.file.inputFile,
 				&s.options,
 				s.fs,
-				originalExt,
 				/* avoidIndex */ false,
 				/* customFilePath */ "",
 			)
 
 			// Apply the asset path template
+			templateExt := strings.TrimPrefix(originalExt, ".")
 			relPath := config.TemplateToString(config.SubstituteTemplate(s.options.AssetPathTemplate, config.PathPlaceholders{
 				Dir:  &dir,
 				Name: &base,
 				Hash: &hash,
-			})) + ext
+				Ext:  &templateExt,
+			})) + originalExt
 
 			// Optionally add metadata about the file
 			var jsonMetadataChunk string
@@ -1873,8 +1906,8 @@ func (s *scanner) validateTLA(sourceIndex uint32) tlaCheck {
 							if parentRepr.AST.TopLevelAwaitKeyword.Len > 0 {
 								tlaPrettyPath = parentResult.file.inputFile.Source.PrettyPath
 								tracker := logger.MakeLineColumnTracker(&parentResult.file.inputFile.Source)
-								notes = append(notes, logger.RangeData(&tracker, parentRepr.AST.TopLevelAwaitKeyword,
-									fmt.Sprintf("The top-level await in %q is here", tlaPrettyPath)))
+								notes = append(notes, tracker.MsgData(parentRepr.AST.TopLevelAwaitKeyword,
+									fmt.Sprintf("The top-level await in %q is here:", tlaPrettyPath)))
 								break
 							}
 
@@ -1886,9 +1919,9 @@ func (s *scanner) validateTLA(sourceIndex uint32) tlaCheck {
 							otherSourceIndex = parentResult.tlaCheck.parent.GetIndex()
 
 							tracker := logger.MakeLineColumnTracker(&parentResult.file.inputFile.Source)
-							notes = append(notes, logger.RangeData(&tracker,
+							notes = append(notes, tracker.MsgData(
 								parentRepr.AST.ImportRecords[parent.importRecordIndex].Range,
-								fmt.Sprintf("The file %q imports the file %q here",
+								fmt.Sprintf("The file %q imports the file %q here:",
 									parentResult.file.inputFile.Source.PrettyPath, s.results[otherSourceIndex].file.inputFile.Source.PrettyPath)))
 						}
 
@@ -1904,7 +1937,7 @@ func (s *scanner) validateTLA(sourceIndex uint32) tlaCheck {
 						}
 
 						tracker := logger.MakeLineColumnTracker(&result.file.inputFile.Source)
-						s.log.AddRangeErrorWithNotes(&tracker, record.Range, text, notes)
+						s.log.AddWithNotes(logger.Error, &tracker, record.Range, text, notes)
 					}
 				}
 			}
@@ -2056,7 +2089,7 @@ func (b *Bundle) Compile(log logger.Log, options config.Options, timer *helpers.
 					case logger.GoAPI:
 						hint = " (use \"AllowOverwrite: true\" to allow this)"
 					}
-					log.AddError(nil, logger.Loc{},
+					log.Add(logger.Error, nil, logger.Range{},
 						fmt.Sprintf("Refusing to overwrite input file %q%s",
 							b.files[sourceIndex].inputFile.Source.PrettyPath, hint))
 				}
@@ -2093,7 +2126,7 @@ func (b *Bundle) Compile(log logger.Log, options config.Options, timer *helpers.
 			if relPath, ok := b.fs.Rel(b.fs.Cwd(), outputPath); ok {
 				outputPath = relPath
 			}
-			log.AddError(nil, logger.Loc{}, "Two output files share the same path but have different contents: "+outputPath)
+			log.Add(logger.Error, nil, logger.Range{}, "Two output files share the same path but have different contents: "+outputPath)
 		}
 		outputFiles = outputFiles[:end]
 	}
