@@ -5,6 +5,7 @@ import (
 
 	"github.com/evanw/esbuild/internal/compat"
 	"github.com/evanw/esbuild/internal/config"
+	"github.com/evanw/esbuild/internal/js_ast"
 )
 
 var ts_suite = suite{
@@ -259,8 +260,8 @@ func TestTSImportMissingES6(t *testing.T) {
 			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
 		},
-		expectedCompileLog: `entry.ts: error: No matching export in "foo.js" for import "default"
-entry.ts: error: No matching export in "foo.js" for import "y"
+		expectedCompileLog: `entry.ts: ERROR: No matching export in "foo.js" for import "default"
+entry.ts: ERROR: No matching export in "foo.js" for import "y"
 `,
 	})
 }
@@ -319,7 +320,7 @@ func TestTSImportMissingFile(t *testing.T) {
 			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
 		},
-		expectedScanLog: `entry.ts: error: Could not resolve "./doesNotExist.ts"
+		expectedScanLog: `entry.ts: ERROR: Could not resolve "./doesNotExist.ts"
 `,
 	})
 }
@@ -930,10 +931,10 @@ func TestTSImplicitExtensionsMissing(t *testing.T) {
 			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.js",
 		},
-		expectedScanLog: `entry.ts: error: Could not resolve "./mjs.mjs"
-entry.ts: error: Could not resolve "./cjs.cjs"
-entry.ts: error: Could not resolve "./js.js"
-entry.ts: error: Could not resolve "./jsx.jsx"
+		expectedScanLog: `entry.ts: ERROR: Could not resolve "./mjs.mjs"
+entry.ts: ERROR: Could not resolve "./cjs.cjs"
+entry.ts: ERROR: Could not resolve "./js.js"
+entry.ts: ERROR: Could not resolve "./jsx.jsx"
 `,
 	})
 }
@@ -1297,3 +1298,276 @@ func TestTSImportCTS(t *testing.T) {
 		},
 	})
 }
+
+func TestTSSideEffectsFalseWarningTypeDeclarations(t *testing.T) {
+	default_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.ts": `
+				import "some-js"
+				import "some-ts"
+				import "empty-js"
+				import "empty-ts"
+				import "empty-dts"
+			`,
+
+			"/node_modules/some-js/package.json": `{ "main": "./foo.js", "sideEffects": false }`,
+			"/node_modules/some-js/foo.js":       `console.log('foo')`,
+
+			"/node_modules/some-ts/package.json": `{ "main": "./foo.ts", "sideEffects": false }`,
+			"/node_modules/some-ts/foo.ts":       `console.log('foo' as string)`,
+
+			"/node_modules/empty-js/package.json": `{ "main": "./foo.js", "sideEffects": false }`,
+			"/node_modules/empty-js/foo.js":       ``,
+
+			"/node_modules/empty-ts/package.json": `{ "main": "./foo.ts", "sideEffects": false }`,
+			"/node_modules/empty-ts/foo.ts":       `export type Foo = number`,
+
+			"/node_modules/empty-dts/package.json": `{ "main": "./foo.d.ts", "sideEffects": false }`,
+			"/node_modules/empty-dts/foo.d.ts":     `export type Foo = number`,
+		},
+		entryPaths: []string{"/entry.ts"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+		},
+		expectedScanLog: `entry.ts: WARNING: Ignoring this import because "node_modules/some-js/foo.js" was marked as having no side effects
+node_modules/some-js/package.json: NOTE: "sideEffects" is false in the enclosing "package.json" file
+entry.ts: WARNING: Ignoring this import because "node_modules/some-ts/foo.ts" was marked as having no side effects
+node_modules/some-ts/package.json: NOTE: "sideEffects" is false in the enclosing "package.json" file
+`,
+	})
+}
+
+func TestTSSiblingNamespace(t *testing.T) {
+	ts_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/let.ts": `
+				export namespace x { export let y = 123 }
+				export namespace x { export let z = y }
+			`,
+			"/function.ts": `
+				export namespace x { export function y() {} }
+				export namespace x { export let z = y }
+			`,
+			"/class.ts": `
+				export namespace x { export class y {} }
+				export namespace x { export let z = y }
+			`,
+			"/namespace.ts": `
+				export namespace x { export namespace y { 0 } }
+				export namespace x { export let z = y }
+			`,
+			"/enum.ts": `
+				export namespace x { export enum y {} }
+				export namespace x { export let z = y }
+			`,
+		},
+		entryPaths: []string{
+			"/let.ts",
+			"/function.ts",
+			"/class.ts",
+			"/namespace.ts",
+			"/enum.ts",
+		},
+		options: config.Options{
+			Mode:         config.ModePassThrough,
+			AbsOutputDir: "/out",
+		},
+	})
+}
+
+func TestTSSiblingEnum(t *testing.T) {
+	ts_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/number.ts": `
+				export enum x { y, yy = y }
+				export enum x { z = y + 1 }
+
+				declare let y: any, z: any
+				export namespace x { console.log(y, z) }
+				console.log(x.y, x.z)
+			`,
+			"/string.ts": `
+				export enum x { y = 'a', yy = y }
+				export enum x { z = y }
+
+				declare let y: any, z: any
+				export namespace x { console.log(y, z) }
+				console.log(x.y, x.z)
+			`,
+			"/propagation.ts": `
+				export enum a { b = 100 }
+				export enum x {
+					c = a.b,
+					d = c * 2,
+					e = x.d ** 2,
+					f = x['e'] / 4,
+				}
+				export enum x { g = f >> 4 }
+				console.log(a.b, a['b'], x.g, x['g'])
+			`,
+			"/nested-number.ts": `
+				export namespace foo { export enum x { y, yy = y } }
+				export namespace foo { export enum x { z = y + 1 } }
+
+				declare let y: any, z: any
+				export namespace foo.x {
+					console.log(y, z)
+					console.log(x.y, x.z)
+				}
+			`,
+			"/nested-string.ts": `
+				export namespace foo { export enum x { y = 'a', yy = y } }
+				export namespace foo { export enum x { z = y } }
+
+				declare let y: any, z: any
+				export namespace foo.x {
+					console.log(y, z)
+					console.log(x.y, x.z)
+				}
+			`,
+			"/nested-propagation.ts": `
+				export namespace n { export enum a { b = 100 } }
+				export namespace n {
+					export enum x {
+						c = n.a.b,
+						d = c * 2,
+						e = x.d ** 2,
+						f = x['e'] / 4,
+					}
+				}
+				export namespace n {
+					export enum x { g = f >> 4 }
+					console.log(a.b, n.a.b, n['a']['b'], x.g, n.x.g, n['x']['g'])
+				}
+			`,
+		},
+		entryPaths: []string{
+			"/number.ts",
+			"/string.ts",
+			"/propagation.ts",
+			"/nested-number.ts",
+			"/nested-string.ts",
+			"/nested-propagation.ts",
+		},
+		options: config.Options{
+			Mode:         config.ModePassThrough,
+			AbsOutputDir: "/out",
+		},
+	})
+}
+
+func TestTSEnumTreeShaking(t *testing.T) {
+	ts_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/simple-member.ts": `
+				enum x { y = 123 }
+				console.log(x.y)
+			`,
+			"/simple-enum.ts": `
+				enum x { y = 123 }
+				console.log(x)
+			`,
+			"/sibling-member.ts": `
+				enum x { y = 123 }
+				enum x { z = y * 2 }
+				console.log(x.y, x.z)
+			`,
+			"/sibling-enum-before.ts": `
+				console.log(x)
+				enum x { y = 123 }
+				enum x { z = y * 2 }
+			`,
+			"/sibling-enum-middle.ts": `
+				enum x { y = 123 }
+				console.log(x)
+				enum x { z = y * 2 }
+			`,
+			"/sibling-enum-after.ts": `
+				enum x { y = 123 }
+				enum x { z = y * 2 }
+				console.log(x)
+			`,
+			"/namespace-before.ts": `
+				namespace x { console.log(x, y) }
+				enum x { y = 123 }
+			`,
+			"/namespace-after.ts": `
+				enum x { y = 123 }
+				namespace x { console.log(x, y) }
+			`,
+		},
+		entryPaths: []string{
+			"/simple-member.ts",
+			"/simple-enum.ts",
+			"/sibling-member.ts",
+			"/sibling-enum-before.ts",
+			"/sibling-enum-middle.ts",
+			"/sibling-enum-after.ts",
+			"/namespace-before.ts",
+			"/namespace-after.ts",
+		},
+		options: config.Options{
+			Mode:         config.ModeBundle,
+			AbsOutputDir: "/out",
+			OutputFormat: config.FormatESModule,
+		},
+	})
+}
+
+func TestTSEnumJSX(t *testing.T) {
+	ts_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/element.tsx": `
+				export enum Foo { Div = 'div' }
+				console.log(<Foo.Div />)
+			`,
+			"/fragment.tsx": `
+				export enum React { Fragment = 'div' }
+				console.log(<>test</>)
+			`,
+			"/nested-element.tsx": `
+				namespace x.y { export enum Foo { Div = 'div' } }
+				namespace x.y { console.log(<x.y.Foo.Div />) }
+			`,
+			"/nested-fragment.tsx": `
+				namespace x.y { export enum React { Fragment = 'div' } }
+				namespace x.y { console.log(<>test</>) }
+			`,
+		},
+		entryPaths: []string{
+			"/element.tsx",
+			"/fragment.tsx",
+			"/nested-element.tsx",
+			"/nested-fragment.tsx",
+		},
+		options: config.Options{
+			Mode:         config.ModePassThrough,
+			AbsOutputDir: "/out",
+		},
+	})
+}
+
+func TestTSEnumDefine(t *testing.T) {
+	ts_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.ts": `
+				enum a { b = 123, c = d }
+			`,
+		},
+		entryPaths: []string{"/entry.ts"},
+		options: config.Options{
+			Mode:         config.ModePassThrough,
+			AbsOutputDir: "/out",
+			Defines: &config.ProcessedDefines{
+				IdentifierDefines: map[string]config.DefineData{
+					"d": {
+						DefineFunc: func(args config.DefineArgs) js_ast.E {
+							return &js_ast.EIdentifier{Ref: args.FindSymbol(args.Loc, "b")}
+						},
+					},
+				},
+			},
+		},
+	})
+}
