@@ -58,22 +58,92 @@ const (
 func (kind MsgKind) String() string {
 	switch kind {
 	case Error:
-		return "error"
+		return "ERROR"
 	case Warning:
-		return "warning"
+		return "WARNING"
 	case Info:
-		return "info"
+		return "INFO"
 	case Note:
-		return "note"
+		return "NOTE"
 	case Debug:
-		return "debug"
+		return "DEBUG"
 	case Verbose:
-		return "verbose"
+		return "VERBOSE"
 	default:
 		panic("Internal error")
 	}
 }
 
+func (kind MsgKind) Icon() string {
+	// Special-case Windows command prompt, which only supports a few characters
+	if isProbablyWindowsCommandPrompt() {
+		switch kind {
+		case Error:
+			return "X"
+		case Warning:
+			return "▲"
+		case Info:
+			return "►"
+		case Note:
+			return "→"
+		case Debug:
+			return "●"
+		case Verbose:
+			return "♦"
+		default:
+			panic("Internal error")
+		}
+	}
+
+	switch kind {
+	case Error:
+		return "✘"
+	case Warning:
+		return "▲"
+	case Info:
+		return "▶"
+	case Note:
+		return "→"
+	case Debug:
+		return "●"
+	case Verbose:
+		return "⬥"
+	default:
+		panic("Internal error")
+	}
+}
+
+var windowsCommandPrompt struct {
+	mutex         sync.Mutex
+	once          bool
+	isProbablyCMD bool
+}
+
+func isProbablyWindowsCommandPrompt() bool {
+	windowsCommandPrompt.mutex.Lock()
+	defer windowsCommandPrompt.mutex.Unlock()
+
+	if !windowsCommandPrompt.once {
+		windowsCommandPrompt.once = true
+
+		// Assume we are running in Windows Command Prompt if we're on Windows. If
+		// so, we can't use emoji because it won't be supported. Except we can
+		// still use emoji if the WT_SESSION environment variable is present
+		// because that means we're running in the new Windows Terminal instead.
+		if runtime.GOOS == "windows" {
+			windowsCommandPrompt.isProbablyCMD = true
+			for _, env := range os.Environ() {
+				if strings.HasPrefix(env, "WT_SESSION=") {
+					windowsCommandPrompt.isProbablyCMD = false
+					break
+				}
+			}
+		}
+	}
+
+	return windowsCommandPrompt.isProbablyCMD
+}
+
 type Msg struct {
 	PluginName string
 	Kind       MsgKind
@@ -113,6 +183,11 @@ func (r Range) End() int32 {
 	return r.Loc.Start + r.Len
 }
 
+type Span struct {
+	Text  string
+	Range Range
+}
+
 // This type is just so we can use Go's native sort function
 type SortableMsgs []Msg
 
@@ -177,6 +252,22 @@ func (a Path) ComesBeforeInSortedOrder(b Path) bool {
 				(a.Flags == b.Flags && a.IgnoredSuffix < b.IgnoredSuffix)))))
 }
 
+var noColorResult bool
+var noColorOnce sync.Once
+
+func hasNoColorEnvironmentVariable() bool {
+	noColorOnce.Do(func() {
+		for _, key := range os.Environ() {
+			// Read "NO_COLOR" from the environment. This is a convention that some
+			// software follows. See https://no-color.org/ for more information.
+			if strings.HasPrefix(key, "NO_COLOR=") {
+				noColorResult = true
+			}
+		}
+	})
+	return noColorResult
+}
+
 // This has a custom implementation instead of using "filepath.Dir/Base/Ext"
 // because it should work the same on Unix and Windows. These names end up in
 // the generated output and the generated output should not depend on the OS.
@@ -245,6 +336,17 @@ func (s *Source) TextForRange(r Range) string {
 	return s.Contents[r.Loc.Start : r.Loc.Start+r.Len]
 }
 
+func (s *Source) LocBeforeWhitespace(loc Loc) Loc {
+	for loc.Start > 0 {
+		c, width := utf8.DecodeLastRuneInString(s.Contents[:loc.Start])
+		if c != ' ' && c != '\t' && c != '\r' && c != '\n' {
+			break
+		}
+		loc.Start -= int32(width)
+	}
+	return loc
+}
+
 func (s *Source) RangeOfOperatorBefore(loc Loc, op string) Range {
 	text := s.Contents[:loc.Start]
 	index := strings.LastIndex(text, op)
@@ -553,6 +655,20 @@ type Colors struct {
 	Cyan    string
 	Magenta string
 	Yellow  string
+
+	RedBgRed     string
+	RedBgWhite   string
+	GreenBgGreen string
+	GreenBgWhite string
+	BlueBgBlue   string
+	BlueBgWhite  string
+
+	CyanBgCyan       string
+	CyanBgBlack      string
+	MagentaBgMagenta string
+	MagentaBgBlack   string
+	YellowBgYellow   string
+	YellowBgBlack    string
 }
 
 var TerminalColors = Colors{
@@ -568,6 +684,20 @@ var TerminalColors = Colors{
 	Cyan:    "\033[36m",
 	Magenta: "\033[35m",
 	Yellow:  "\033[33m",
+
+	RedBgRed:     "\033[41;31m",
+	RedBgWhite:   "\033[41;97m",
+	GreenBgGreen: "\033[42;32m",
+	GreenBgWhite: "\033[42;97m",
+	BlueBgBlue:   "\033[44;34m",
+	BlueBgWhite:  "\033[44;97m",
+
+	CyanBgCyan:       "\033[46;36m",
+	CyanBgBlack:      "\033[46;30m",
+	MagentaBgMagenta: "\033[45;35m",
+	MagentaBgBlack:   "\033[45;30m",
+	YellowBgYellow:   "\033[43;33m",
+	YellowBgBlack:    "\033[43;30m",
 }
 
 func PrintText(file *os.File, level LogLevel, osArgs []string, callback func(Colors) string) {
@@ -650,23 +780,9 @@ const sizeWarningThreshold = 1024 * 1024
 
 func PrintSummary(useColor UseColor, table SummaryTable, start *time.Time) {
 	PrintTextWithColor(os.Stderr, useColor, func(colors Colors) string {
-		isProbablyWindowsCommandPrompt := false
+		isProbablyWindowsCommandPrompt := isProbablyWindowsCommandPrompt()
 		sb := strings.Builder{}
 
-		// Assume we are running in Windows Command Prompt if we're on Windows. If
-		// so, we can't use emoji because it won't be supported. Except we can
-		// still use emoji if the WT_SESSION environment variable is present
-		// because that means we're running in the new Windows Terminal instead.
-		if runtime.GOOS == "windows" {
-			isProbablyWindowsCommandPrompt = true
-			for _, env := range os.Environ() {
-				if strings.HasPrefix(env, "WT_SESSION=") {
-					isProbablyWindowsCommandPrompt = false
-					break
-				}
-			}
-		}
-
 		if len(table) > 0 {
 			info := GetTerminalInfo(os.Stderr)
 
@@ -807,7 +923,14 @@ func PrintSummary(useColor UseColor, table SummaryTable, start *time.Time) {
 	})
 }
 
-func NewDeferLog() Log {
+type DeferLogKind uint8
+
+const (
+	DeferLogAll DeferLogKind = iota
+	DeferLogNoVerboseOrDebug
+)
+
+func NewDeferLog(kind DeferLogKind) Log {
 	var msgs SortableMsgs
 	var mutex sync.Mutex
 	var hasErrors bool
@@ -816,8 +939,7 @@ func NewDeferLog() Log {
 		Level: LevelInfo,
 
 		AddMsg: func(msg Msg) {
-			if msg.Kind == Verbose || msg.Kind == Debug {
-				// Ignore these when not writing to stderr
+			if kind == DeferLogNoVerboseOrDebug && (msg.Kind == Verbose || msg.Kind == Debug) {
 				return
 			}
 			mutex.Lock()
@@ -862,35 +984,17 @@ type OutputOptions struct {
 }
 
 func (msg Msg) String(options OutputOptions, terminalInfo TerminalInfo) string {
-	// Compute the maximum margin
-	maxMargin := 0
-	if options.IncludeSource {
-		if msg.Data.Location != nil {
-			maxMargin = len(fmt.Sprintf("%d", msg.Data.Location.Line))
-		}
-		for _, note := range msg.Notes {
-			if note.Location != nil {
-				margin := len(fmt.Sprintf("%d", note.Location.Line))
-				if margin > maxMargin {
-					maxMargin = margin
-				}
-			}
-		}
-	}
-
 	// Format the message
-	text := msgString(options.IncludeSource, terminalInfo, msg.Kind, msg.Data, maxMargin, msg.PluginName)
-
-	// Put a blank line between the message and the notes if the message has a stack trace
-	gap := ""
-	if loc := msg.Data.Location; loc != nil && strings.ContainsRune(loc.LineText, '\n') {
-		gap = "\n"
-	}
+	text := msgString(options.IncludeSource, terminalInfo, msg.Kind, msg.Data, msg.PluginName)
 
 	// Format the notes
-	for _, note := range msg.Notes {
-		text += gap
-		text += msgString(options.IncludeSource, terminalInfo, Note, note, maxMargin, "")
+	var oldData MsgData
+	for i, note := range msg.Notes {
+		if options.IncludeSource && (i == 0 || strings.IndexByte(oldData.Text, '\n') >= 0 || oldData.Location != nil) {
+			text += "\n"
+		}
+		text += msgString(options.IncludeSource, terminalInfo, Note, note, "")
+		oldData = note
 	}
 
 	// Add extra spacing between messages if source code is present
@@ -901,109 +1005,226 @@ func (msg Msg) String(options OutputOptions, terminalInfo TerminalInfo) string {
 }
 
 // The number of margin characters in addition to the line number
-const extraMarginChars = 7
+const extraMarginChars = 9
 
 func marginWithLineText(maxMargin int, line int) string {
 	number := fmt.Sprintf("%d", line)
-	return fmt.Sprintf("    %s%s │ ", strings.Repeat(" ", maxMargin-len(number)), number)
+	return fmt.Sprintf("      %s%s │ ", strings.Repeat(" ", maxMargin-len(number)), number)
 }
 
 func emptyMarginText(maxMargin int, isLast bool) string {
 	space := strings.Repeat(" ", maxMargin)
 	if isLast {
-		return fmt.Sprintf("    %s ╵ ", space)
+		return fmt.Sprintf("      %s ╵ ", space)
 	}
-	return fmt.Sprintf("    %s │ ", space)
+	return fmt.Sprintf("      %s │ ", space)
 }
 
-func msgString(includeSource bool, terminalInfo TerminalInfo, kind MsgKind, data MsgData, maxMargin int, pluginName string) string {
+func msgString(includeSource bool, terminalInfo TerminalInfo, kind MsgKind, data MsgData, pluginName string) string {
+	if !includeSource {
+		if loc := data.Location; loc != nil {
+			return fmt.Sprintf("%s: %s: %s\n", loc.File, kind.String(), data.Text)
+		}
+		return fmt.Sprintf("%s: %s\n", kind.String(), data.Text)
+	}
+
 	var colors Colors
 	if terminalInfo.UseColorEscapes {
 		colors = TerminalColors
 	}
 
-	var kindColor string
-	prefixColor := colors.Bold
-	messageColor := colors.Bold
-	textIndent := ""
+	var iconColor string
+	var kindColorBrackets string
+	var kindColorText string
 
-	if includeSource {
-		textIndent = " > "
+	location := ""
+
+	if data.Location != nil {
+		maxMargin := len(fmt.Sprintf("%d", data.Location.Line))
+		d := detailStruct(data, terminalInfo, maxMargin)
+		if d.Suggestion != "" {
+			location = fmt.Sprintf("\n    %s:%d:%d:\n%s%s%s%s%s%s\n%s%s%s%s%s\n%s%s%s%s%s\n%s",
+				d.Path, d.Line, d.Column,
+				colors.Dim, d.SourceBefore, colors.Green, d.SourceMarked, colors.Dim, d.SourceAfter,
+				emptyMarginText(maxMargin, false), d.Indent, colors.Green, d.Marker, colors.Dim,
+				emptyMarginText(maxMargin, true), d.Indent, colors.Green, d.Suggestion, colors.Reset,
+				d.ContentAfter,
+			)
+		} else {
+			location = fmt.Sprintf("\n    %s:%d:%d:\n%s%s%s%s%s%s\n%s%s%s%s%s\n%s",
+				d.Path, d.Line, d.Column,
+				colors.Dim, d.SourceBefore, colors.Green, d.SourceMarked, colors.Dim, d.SourceAfter,
+				emptyMarginText(maxMargin, true), d.Indent, colors.Green, d.Marker, colors.Reset,
+				d.ContentAfter,
+			)
+		}
 	}
 
 	switch kind {
 	case Verbose:
-		kindColor = colors.Cyan
+		iconColor = colors.Cyan
+		kindColorBrackets = colors.CyanBgCyan
+		kindColorText = colors.CyanBgBlack
 
 	case Debug:
-		kindColor = colors.Blue
+		iconColor = colors.Green
+		kindColorBrackets = colors.GreenBgGreen
+		kindColorText = colors.GreenBgWhite
 
 	case Info:
-		kindColor = colors.Green
+		iconColor = colors.Blue
+		kindColorBrackets = colors.BlueBgBlue
+		kindColorText = colors.BlueBgWhite
 
 	case Error:
-		kindColor = colors.Red
+		iconColor = colors.Red
+		kindColorBrackets = colors.RedBgRed
+		kindColorText = colors.RedBgWhite
 
 	case Warning:
-		kindColor = colors.Magenta
+		iconColor = colors.Yellow
+		kindColorBrackets = colors.YellowBgYellow
+		kindColorText = colors.YellowBgBlack
 
 	case Note:
-		prefixColor = colors.Reset
-		kindColor = colors.Bold
-		messageColor = ""
-		if includeSource {
-			textIndent = "   "
+		sb := strings.Builder{}
+
+		for _, line := range strings.Split(data.Text, "\n") {
+			// Special-case word wrapping
+			if wrapWidth := terminalInfo.Width; wrapWidth > 2 {
+				if wrapWidth > 100 {
+					wrapWidth = 100 // Enforce a maximum paragraph width for readability
+				}
+				for _, run := range wrapWordsInString(line, wrapWidth-2) {
+					sb.WriteString("  ")
+					sb.WriteString(linkifyText(run, colors.Underline, colors.Reset))
+					sb.WriteByte('\n')
+				}
+				continue
+			}
+
+			// Otherwise, just write an indented line
+			sb.WriteString("  ")
+			sb.WriteString(linkifyText(line, colors.Underline, colors.Reset))
+			sb.WriteByte('\n')
 		}
 
-	default:
-		panic("Internal error")
+		sb.WriteString(location)
+		return sb.String()
 	}
 
-	var pluginText string
-	if pluginName != "" {
-		pluginText = fmt.Sprintf("%s[plugin: %s] ", colors.Yellow, pluginName)
+	return fmt.Sprintf("%s%s %s[%s%s%s]%s %s%s%s\n%s",
+		iconColor, kind.Icon(),
+		kindColorBrackets, kindColorText, kind.String(), kindColorBrackets, colors.Reset,
+		colors.Bold, data.Text, colors.Reset,
+		location,
+	)
+}
+
+func linkifyText(text string, underline string, reset string) string {
+	if underline == "" {
+		return text
 	}
 
-	if data.Location == nil {
-		return fmt.Sprintf("%s%s%s%s: %s%s%s%s\n%s",
-			prefixColor, textIndent, kindColor, kind.String(),
-			pluginText, colors.Reset, messageColor, data.Text,
-			colors.Reset)
+	https := strings.Index(text, "https://")
+	if https == -1 {
+		return text
 	}
 
-	if !includeSource {
-		return fmt.Sprintf("%s%s%s: %s%s: %s%s%s%s\n%s",
-			prefixColor, textIndent, data.Location.File,
-			kindColor, kind.String(),
-			pluginText, colors.Reset, messageColor, data.Text,
-			colors.Reset)
+	sb := strings.Builder{}
+	for {
+		https := strings.Index(text, "https://")
+		if https == -1 {
+			break
+		}
+
+		end := strings.IndexByte(text[https:], ' ')
+		if end == -1 {
+			end = len(text)
+		} else {
+			end += https
+		}
+
+		// Remove trailing punctuation
+		if end > https {
+			switch text[end-1] {
+			case '.', ',', '?', '!', ')', ']', '}':
+				end--
+			}
+		}
+
+		sb.WriteString(text[:https])
+		sb.WriteString(underline)
+		sb.WriteString(text[https:end])
+		sb.WriteString(reset)
+		text = text[end:]
 	}
 
-	d := detailStruct(data, terminalInfo, maxMargin)
+	sb.WriteString(text)
+	return sb.String()
+}
 
-	callout := d.Marker
-	calloutPrefix := ""
+func wrapWordsInString(text string, width int) []string {
+	runs := []string{}
 
-	if d.Suggestion != "" {
-		callout = d.Suggestion
-		calloutPrefix = fmt.Sprintf("%s%s%s%s%s\n",
-			emptyMarginText(maxMargin, false), d.Indent, colors.Green, d.Marker, colors.Dim)
+outer:
+	for text != "" {
+		i := 0
+		x := 0
+		wordEndI := 0
+
+		// Skip over any leading spaces
+		for i < len(text) && text[i] == ' ' {
+			i++
+			x++
+		}
+
+		// Find out how many words will fit in this run
+		for i < len(text) {
+			oldWordEndI := wordEndI
+			wordStartI := i
+
+			// Find the end of the word
+			for i < len(text) {
+				c, width := utf8.DecodeRuneInString(text[i:])
+				if c == ' ' {
+					break
+				}
+				i += width
+				x += 1 // Naively assume that each unicode code point is a single column
+			}
+			wordEndI = i
+
+			// Split into a new run if this isn't the first word in the run and the end is past the width
+			if wordStartI > 0 && x > width {
+				runs = append(runs, text[:oldWordEndI])
+				text = text[wordStartI:]
+				continue outer
+			}
+
+			// Skip over any spaces after the word
+			for i < len(text) && text[i] == ' ' {
+				i++
+				x++
+			}
+		}
+
+		// If we get here, this is the last run (i.e. everything fits)
+		break
 	}
 
-	return fmt.Sprintf("%s%s%s:%d:%d: %s%s: %s%s%s%s\n%s%s%s%s%s%s%s\n%s%s%s%s%s%s%s\n%s",
-		prefixColor, textIndent, d.Path, d.Line, d.Column,
-		kindColor, kind.String(),
-		pluginText, colors.Reset, messageColor, d.Message,
-		colors.Reset, colors.Dim, d.SourceBefore, colors.Green, d.SourceMarked, colors.Dim, d.SourceAfter,
-		calloutPrefix, emptyMarginText(maxMargin, true), d.Indent, colors.Green, callout, colors.Dim, d.ContentAfter,
-		colors.Reset)
+	// Remove any trailing spaces on the last run
+	for len(text) > 0 && text[len(text)-1] == ' ' {
+		text = text[:len(text)-1]
+	}
+	runs = append(runs, text)
+	return runs
 }
 
 type MsgDetail struct {
-	Path    string
-	Line    int
-	Column  int
-	Message string
+	Path   string
+	Line   int
+	Column int
 
 	SourceBefore string
 	SourceMarked string
@@ -1016,59 +1237,171 @@ type MsgDetail struct {
 	ContentAfter string
 }
 
-func computeLineAndColumn(contents string, offset int) (lineCount int, columnCount int, lineStart int, lineEnd int) {
-	var prevCodePoint rune
-	if offset > len(contents) {
-		offset = len(contents)
-	}
-
-	// Scan up to the offset and count lines
-	for i, codePoint := range contents[:offset] {
-		switch codePoint {
-		case '\n':
-			lineStart = i + 1
-			if prevCodePoint != '\r' {
-				lineCount++
-			}
-		case '\r':
-			lineStart = i + 1
-			lineCount++
-		case '\u2028', '\u2029':
-			lineStart = i + 3 // These take three bytes to encode in UTF-8
-			lineCount++
-		}
-		prevCodePoint = codePoint
-	}
-
-	// Scan to the end of the line (or end of file if this is the last line)
-	lineEnd = len(contents)
-loop:
-	for i, codePoint := range contents[offset:] {
-		switch codePoint {
-		case '\r', '\n', '\u2028', '\u2029':
-			lineEnd = offset + i
-			break loop
-		}
-	}
-
-	columnCount = offset - lineStart
-	return
+// It's not common for large files to have many warnings. But when it happens,
+// we want to make sure that it's not too slow. Source code locations are
+// represented as byte offsets for compactness but transforming these to
+// line/column locations for warning messages requires scanning through the
+// file. A naive approach for this would cause O(n^2) scanning time for n
+// warnings distributed throughout the file.
+//
+// Warnings are typically generated sequentially as the file is scanned. So
+// one way of optimizing this is to just start scanning from where we left
+// off last time instead of always starting from the beginning of the file.
+// That's what this object does.
+//
+// Another option could be to eagerly populate an array of line/column offsets
+// and then use binary search for each query. This might slow down the common
+// case of a file with only at most a few warnings though, so think before
+// optimizing too much. Performance in the zero or one warning case is by far
+// the most important.
+type LineColumnTracker struct {
+	contents     string
+	prettyPath   string
+	offset       int32
+	line         int32
+	lineStart    int32
+	lineEnd      int32
+	hasLineStart bool
+	hasLineEnd   bool
+	hasSource    bool
 }
 
-func LocationOrNil(source *Source, r Range) *MsgLocation {
+func MakeLineColumnTracker(source *Source) LineColumnTracker {
 	if source == nil {
+		return LineColumnTracker{
+			hasSource: false,
+		}
+	}
+
+	return LineColumnTracker{
+		contents:     source.Contents,
+		prettyPath:   source.PrettyPath,
+		hasLineStart: true,
+		hasSource:    true,
+	}
+}
+
+func (tracker *LineColumnTracker) MsgData(r Range, text string) MsgData {
+	return MsgData{
+		Text:     text,
+		Location: tracker.MsgLocationOrNil(r),
+	}
+}
+
+func (t *LineColumnTracker) scanTo(offset int32) {
+	contents := t.contents
+	i := t.offset
+
+	// Scan forward
+	if i < offset {
+		for {
+			r, size := utf8.DecodeRuneInString(contents[i:])
+			i += int32(size)
+
+			switch r {
+			case '\n':
+				t.hasLineStart = true
+				t.hasLineEnd = false
+				t.lineStart = i
+				if i == int32(size) || contents[i-int32(size)-1] != '\r' {
+					t.line++
+				}
+
+			case '\r', '\u2028', '\u2029':
+				t.hasLineStart = true
+				t.hasLineEnd = false
+				t.lineStart = i
+				t.line++
+			}
+
+			if i >= offset {
+				t.offset = i
+				return
+			}
+		}
+	}
+
+	// Scan backward
+	if i > offset {
+		for {
+			r, size := utf8.DecodeLastRuneInString(contents[:i])
+			i -= int32(size)
+
+			switch r {
+			case '\n':
+				t.hasLineStart = false
+				t.hasLineEnd = true
+				t.lineEnd = i
+				if i == 0 || contents[i-1] != '\r' {
+					t.line--
+				}
+
+			case '\r', '\u2028', '\u2029':
+				t.hasLineStart = false
+				t.hasLineEnd = true
+				t.lineEnd = i
+				t.line--
+			}
+
+			if i <= offset {
+				t.offset = i
+				return
+			}
+		}
+	}
+}
+
+func (t *LineColumnTracker) computeLineAndColumn(offset int) (lineCount int, columnCount int, lineStart int, lineEnd int) {
+	t.scanTo(int32(offset))
+
+	// Scan for the start of the line
+	if !t.hasLineStart {
+		contents := t.contents
+		i := t.offset
+		for i > 0 {
+			r, size := utf8.DecodeLastRuneInString(contents[:i])
+			if r == '\n' || r == '\r' || r == '\u2028' || r == '\u2029' {
+				break
+			}
+			i -= int32(size)
+		}
+		t.hasLineStart = true
+		t.lineStart = i
+	}
+
+	// Scan for the end of the line
+	if !t.hasLineEnd {
+		contents := t.contents
+		i := t.offset
+		n := int32(len(contents))
+		for i < n {
+			r, size := utf8.DecodeRuneInString(contents[i:])
+			if r == '\n' || r == '\r' || r == '\u2028' || r == '\u2029' {
+				break
+			}
+			i += int32(size)
+		}
+		t.hasLineEnd = true
+		t.lineEnd = i
+	}
+
+	return int(t.line), offset - int(t.lineStart), int(t.lineStart), int(t.lineEnd)
+}
+
+func (tracker *LineColumnTracker) MsgLocationOrNil(r Range) *MsgLocation {
+	if tracker == nil || !tracker.hasSource {
 		return nil
 	}
 
 	// Convert the index into a line and column number
-	lineCount, columnCount, lineStart, lineEnd := computeLineAndColumn(source.Contents, int(r.Loc.Start))
+	lineCount, columnCount, lineStart, lineEnd := tracker.computeLineAndColumn(int(r.Loc.Start))
 
 	return &MsgLocation{
-		File:     source.PrettyPath,
+		File:     tracker.prettyPath,
 		Line:     lineCount + 1, // 0-based to 1-based
 		Column:   columnCount,
 		Length:   int(r.Len),
-		LineText: source.Contents[lineStart:lineEnd],
+		LineText: tracker.contents[lineStart:lineEnd],
 	}
 }
 
@@ -1197,10 +1530,9 @@ func detailStruct(data MsgData, terminalInfo TerminalInfo, maxMargin int) MsgDet
 	margin := marginWithLineText(maxMargin, loc.Line)
 
 	return MsgDetail{
-		Path:    loc.File,
-		Line:    loc.Line,
-		Column:  loc.Column,
-		Message: data.Text,
+		Path:   loc.File,
+		Line:   loc.Line,
+		Column: loc.Column,
 
 		SourceBefore: margin + lineText[:markerStart],
 		SourceMarked: lineText[markerStart:markerEnd],
@@ -1255,98 +1587,17 @@ func renderTabStops(withTabs string, spacesPerTab int) string {
 	return withoutTabs.String()
 }
 
-func (log Log) AddError(source *Source, loc Loc, text string) {
+func (log Log) Add(kind MsgKind, tracker *LineColumnTracker, r Range, text string) {
 	log.AddMsg(Msg{
-		Kind: Error,
-		Data: RangeData(source, Range{Loc: loc}, text),
+		Kind: kind,
+		Data: tracker.MsgData(r, text),
 	})
 }
 
-func (log Log) AddErrorWithNotes(source *Source, loc Loc, text string, notes []MsgData) {
+func (log Log) AddWithNotes(kind MsgKind, tracker *LineColumnTracker, r Range, text string, notes []MsgData) {
 	log.AddMsg(Msg{
-		Kind:  Error,
-		Data:  RangeData(source, Range{Loc: loc}, text),
+		Kind:  kind,
+		Data:  tracker.MsgData(r, text),
 		Notes: notes,
 	})
 }
-
-func (log Log) AddWarning(source *Source, loc Loc, text string) {
-	log.AddMsg(Msg{
-		Kind: Warning,
-		Data: RangeData(source, Range{Loc: loc}, text),
-	})
-}
-
-func (log Log) AddDebug(source *Source, loc Loc, text string) {
-	log.AddMsg(Msg{
-		Kind: Debug,
-		Data: RangeData(source, Range{Loc: loc}, text),
-	})
-}
-
-func (log Log) AddDebugWithNotes(source *Source, loc Loc, text string, notes []MsgData) {
-	log.AddMsg(Msg{
-		Kind:  Debug,
-		Data:  RangeData(source, Range{Loc: loc}, text),
-		Notes: notes,
-	})
-}
-
-func (log Log) AddVerbose(source *Source, loc Loc, text string) {
-	log.AddMsg(Msg{
-		Kind: Verbose,
-		Data: RangeData(source, Range{Loc: loc}, text),
-	})
-}
-
-func (log Log) AddVerboseWithNotes(source *Source, loc Loc, text string, notes []MsgData) {
-	log.AddMsg(Msg{
-		Kind:  Verbose,
-		Data:  RangeData(source, Range{Loc: loc}, text),
-		Notes: notes,
-	})
-}
-
-func (log Log) AddRangeError(source *Source, r Range, text string) {
-	log.AddMsg(Msg{
-		Kind: Error,
-		Data: RangeData(source, r, text),
-	})
-}
-
-func (log Log) AddRangeWarning(source *Source, r Range, text string) {
-	log.AddMsg(Msg{
-		Kind: Warning,
-		Data: RangeData(source, r, text),
-	})
-}
-
-func (log Log) AddRangeDebug(source *Source, r Range, text string) {
-	log.AddMsg(Msg{
-		Kind: Debug,
-		Data: RangeData(source, r, text),
-	})
-}
-
-func (log Log) AddRangeErrorWithNotes(source *Source, r Range, text string, notes []MsgData) {
-	log.AddMsg(Msg{
-		Kind:  Error,
-		Data:  RangeData(source, r, text),
-		Notes: notes,
-	})
-}
-
-func (log Log) AddRangeWarningWithNotes(source *Source, r Range, text string, notes []MsgData) {
-	log.AddMsg(Msg{
-		Kind:  Warning,
-		Data:  RangeData(source, r, text),
-		Notes: notes,
-	})
-}
-
-func RangeData(source *Source, r Range, text string) MsgData {
-	return MsgData{
-		Text:     text,
-		Location: LocationOrNil(source, r),
-	}
-}
