@@ -17,7 +17,7 @@ import (
 
 type packageJSON struct {
 	source     logger.Source
-	mainFields map[string]string
+	mainFields map[string]mainField
 	moduleType config.ModuleType
 
 	// Present if the "browser" field is present. This field is intended to be
@@ -68,6 +68,11 @@ type packageJSON struct {
 	exportsMap *pjMap
 }
 
+type mainField struct {
+	keyLoc  logger.Loc
+	relPath string
+}
+
 type browserPathKind uint8
 
 const (
@@ -216,7 +221,7 @@ func (r resolverQuery) parsePackageJSON(inputPath string) *packageJSON {
 		r.debugLogs.addNote(fmt.Sprintf("Failed to read file %q: %s", packageJSONPath, originalError.Error()))
 	}
 	if err != nil {
-		r.log.AddError(nil, logger.Loc{},
+		r.log.Add(logger.Error, nil, logger.Range{},
 			fmt.Sprintf("Cannot read file %q: %s",
 				r.PrettyPath(logger.Path{Text: packageJSONPath, Namespace: "file"}), err.Error()))
 		return nil
@@ -238,7 +243,10 @@ func (r resolverQuery) parsePackageJSON(inputPath string) *packageJSON {
 		return nil
 	}
 
-	packageJSON := &packageJSON{source: jsonSource}
+	packageJSON := &packageJSON{
+		source:     jsonSource,
+		mainFields: make(map[string]mainField),
+	}
 
 	// Read the "type" field
 	if typeJSON, _, ok := getProperty(json, "type"); ok {
@@ -249,11 +257,13 @@ func (r resolverQuery) parsePackageJSON(inputPath string) *packageJSON {
 			case "module":
 				packageJSON.moduleType = config.ModuleESM
 			default:
-				r.log.AddRangeWarning(&tracker, jsonSource.RangeOfString(typeJSON.Loc),
-					fmt.Sprintf("%q is not a valid value for the \"type\" field (must be either \"commonjs\" or \"module\")", typeValue))
+				r.log.AddWithNotes(logger.Warning, &tracker, jsonSource.RangeOfString(typeJSON.Loc),
+					fmt.Sprintf("%q is not a valid value for the \"type\" field", typeValue),
+					[]logger.MsgData{{Text: "The \"type\" field must be set to either \"commonjs\" or \"module\"."}},
+				)
 			}
 		} else {
-			r.log.AddWarning(&tracker, typeJSON.Loc,
+			r.log.Add(logger.Warning, &tracker, logger.Range{Loc: typeJSON.Loc},
 				"The value for \"type\" must be a string")
 		}
 	}
@@ -264,12 +274,18 @@ func (r resolverQuery) parsePackageJSON(inputPath string) *packageJSON {
 		mainFields = defaultMainFields[r.options.Platform]
 	}
 	for _, field := range mainFields {
-		if mainJSON, _, ok := getProperty(json, field); ok {
+		if mainJSON, mainRange, ok := getProperty(json, field); ok {
 			if main, ok := getString(mainJSON); ok && main != "" {
-				if packageJSON.mainFields == nil {
-					packageJSON.mainFields = make(map[string]string)
+				packageJSON.mainFields[field] = mainField{mainRange, main}
+			}
+		}
+	}
+	for _, field := range mainFieldsForFailure {
+		if _, ok := packageJSON.mainFields[field]; !ok {
+			if mainJSON, mainRange, ok := getProperty(json, field); ok {
+				if main, ok := getString(mainJSON); ok && main != "" {
+					packageJSON.mainFields[field] = mainField{mainRange, main}
 				}
-				packageJSON.mainFields[field] = main
 			}
 		}
 	}
@@ -303,7 +319,7 @@ func (r resolverQuery) parsePackageJSON(inputPath string) *packageJSON {
 							browserMap[key] = nil
 						}
 					} else {
-						r.log.AddWarning(&tracker, prop.ValueOrNil.Loc,
+						r.log.Add(logger.Warning, &tracker, logger.Range{Loc: prop.ValueOrNil.Loc},
 							"Each \"browser\" mapping must be a string or a boolean")
 					}
 				}
@@ -340,7 +356,7 @@ func (r resolverQuery) parsePackageJSON(inputPath string) *packageJSON {
 			for _, itemJSON := range data.Items {
 				item, ok := itemJSON.Data.(*js_ast.EString)
 				if !ok || item.Value == nil {
-					r.log.AddWarning(&tracker, itemJSON.Loc,
+					r.log.Add(logger.Warning, &tracker, logger.Range{Loc: itemJSON.Loc},
 						"Expected string in array for \"sideEffects\"")
 					continue
 				}
@@ -364,7 +380,7 @@ func (r resolverQuery) parsePackageJSON(inputPath string) *packageJSON {
 			}
 
 		default:
-			r.log.AddWarning(&tracker, sideEffectsJSON.Loc,
+			r.log.Add(logger.Warning, &tracker, logger.Range{Loc: sideEffectsJSON.Loc},
 				"The value for \"sideEffects\" must be a boolean or an array")
 		}
 	}
@@ -373,7 +389,7 @@ func (r resolverQuery) parsePackageJSON(inputPath string) *packageJSON {
 	if importsJSON, _, ok := getProperty(json, "imports"); ok {
 		if importsMap := parseImportsExportsMap(jsonSource, r.log, importsJSON); importsMap != nil {
 			if importsMap.root.kind != pjObject {
-				r.log.AddRangeWarning(&tracker, importsMap.root.firstToken,
+				r.log.Add(logger.Warning, &tracker, importsMap.root.firstToken,
 					"The value for \"imports\" must be an object")
 			}
 			packageJSON.importsMap = importsMap
@@ -549,10 +565,10 @@ func parseImportsExportsMap(source logger.Source, log logger.Log, json js_ast.Ex
 					isConditionalSugar = curIsConditionalSugar
 				} else if isConditionalSugar != curIsConditionalSugar {
 					prevEntry := mapData[i-1]
-					log.AddRangeWarningWithNotes(&tracker, keyRange,
+					log.AddWithNotes(logger.Warning, &tracker, keyRange,
 						"This object cannot contain keys that both start with \".\" and don't start with \".\"",
-						[]logger.MsgData{logger.RangeData(&tracker, prevEntry.keyRange,
-							fmt.Sprintf("The previous key %q is incompatible with the current key %q", prevEntry.key, key))})
+						[]logger.MsgData{tracker.MsgData(prevEntry.keyRange,
+							fmt.Sprintf("The key %q is incompatible with the previous key %q:", key, prevEntry.key))})
 					return pjEntry{
 						kind:       pjInvalid,
 						firstToken: firstToken,
@@ -593,7 +609,8 @@ func parseImportsExportsMap(source logger.Source, log logger.Log, json js_ast.Ex
 			firstToken.Loc = expr.Loc
 		}
 
-		log.AddRangeWarning(&tracker, firstToken, "This value must be a string, an object, an array, or null")
+		log.Add(logger.Warning, &tracker, firstToken,
+			"This value must be a string, an object, an array, or null")
 		return pjEntry{
 			kind:       pjInvalid,
 			firstToken: firstToken,
