@@ -19,6 +19,9 @@ export const transform: typeof types.transform = (input, options) =>
 export const formatMessages: typeof types.formatMessages = (messages, options) =>
   ensureServiceIsRunning().formatMessages(messages, options);
 
+export const analyzeMetafile: typeof types.analyzeMetafile = (metafile, options) =>
+  ensureServiceIsRunning().analyzeMetafile(metafile, options);
+
 export const buildSync: typeof types.buildSync = () => {
   throw new Error(`The "buildSync" API only works in node`);
 };
@@ -31,10 +34,15 @@ export const formatMessagesSync: typeof types.formatMessagesSync = () => {
   throw new Error(`The "formatMessagesSync" API only works in node`);
 };
 
+export const analyzeMetafileSync: typeof types.analyzeMetafileSync = () => {
+  throw new Error(`The "analyzeMetafileSync" API only works in node`);
+};
+
 interface Service {
   build: typeof types.build;
   transform: typeof types.transform;
   formatMessages: typeof types.formatMessages;
+  analyzeMetafile: typeof types.analyzeMetafile;
 }
 
 let initializePromise: Promise<void> | undefined;
@@ -141,5 +149,14 @@ const startRunningService = async (wasmURL: string, useWorker: boolean): Promise
           options,
           callback: (err, res) => err ? reject(err) : resolve(res!),
         })),
+    analyzeMetafile: (metafile, options) =>
+      new Promise((resolve, reject) =>
+        service.analyzeMetafile({
+          callName: 'analyzeMetafile',
+          refs: null,
+          metafile: typeof metafile === 'string' ? metafile : JSON.stringify(metafile),
+          options,
+          callback: (err, res) => err ? reject(err) : resolve(res!),
+        })),
   }
 }
