@@ -1,5 +1,6 @@
 import * as types from "../shared/types";
 import * as common from "../shared/common";
+import { ESBUILD_BINARY_PATH, generateBinPath } from "./node-platform";
 
 import child_process = require('child_process');
 import crypto = require('crypto');
@@ -45,15 +46,9 @@ if (process.env.ESBUILD_WORKER_THREADS !== '0') {
 let isInternalWorkerThread = worker_threads?.workerData?.esbuildVersion === ESBUILD_VERSION;
 
 let esbuildCommandAndArgs = (): [string, string[]] => {
-  // This feature was added to give external code a way to modify the binary
-  // path without modifying the code itself. Do not remove this because
-  // external code relies on this.
-  if (process.env.ESBUILD_BINARY_PATH) {
-    return [path.resolve(process.env.ESBUILD_BINARY_PATH), []];
-  }
-
   // Try to have a nice error message when people accidentally bundle esbuild
-  if (path.basename(__filename) !== 'main.js' || path.basename(__dirname) !== 'lib') {
+  // without providing an explicit path to the binary, or when using WebAssembly.
+  if ((!ESBUILD_BINARY_PATH || WASM) && (path.basename(__filename) !== 'main.js' || path.basename(__dirname) !== 'lib')) {
     throw new Error(
       `The esbuild JavaScript API cannot be bundled. Please mark the "esbuild" ` +
       `package as external so it's not included in the bundle.\n` +
@@ -72,33 +67,7 @@ let esbuildCommandAndArgs = (): [string, string[]] => {
     return ['node', [path.join(__dirname, '..', 'bin', 'esbuild')]];
   }
 
-  if (process.platform === 'win32') {
-    return [path.join(__dirname, '..', 'esbuild.exe'), []];
-  }
-
-  // Yarn 2 is deliberately incompatible with binary modules because the
-  // developers of Yarn 2 don't think they should be used. See this thread for
-  // details: https://github.com/yarnpkg/berry/issues/882.
-  //
-  // As a compatibility hack we replace the binary with a wrapper script only
-  // for Yarn 2. The wrapper script is avoided for other platforms because
-  // running the binary directly without going through node first is faster.
-  // However, this will make using the JavaScript API with Yarn 2 unnecessarily
-  // slow because the wrapper means running the binary will now start another
-  // nested node process just to call "spawnSync" and run the actual binary.
-  //
-  // To work around this workaround, we query for the place the binary is moved
-  // to if the original location is replaced by our Yarn 2 compatibility hack.
-  // If it exists, we can infer that we are running within Yarn 2 and the
-  // JavaScript API should invoke the binary here instead to avoid a slowdown.
-  // Calling the binary directly can be over 6x faster than calling the wrapper
-  // script instead.
-  let pathForYarn2 = path.join(__dirname, '..', 'esbuild');
-  if (fs.existsSync(pathForYarn2)) {
-    return [pathForYarn2, []];
-  }
-
-  return [path.join(__dirname, '..', 'bin', 'esbuild'), []];
+  return [generateBinPath(), []];
 };
 
 // Return true if stderr is a TTY
@@ -113,7 +82,7 @@ let fsSync: common.StreamFS = {
       } catch {
       }
       callback(null, contents);
-    } catch (err) {
+    } catch (err: any) {
       callback(err, null);
     }
   },
@@ -138,7 +107,7 @@ let fsAsync: common.StreamFS = {
           callback(err, contents);
         }
       });
-    } catch (err) {
+    } catch (err: any) {
       callback(err, null);
     }
   },
@@ -167,6 +136,9 @@ export let transform: typeof types.transform = (input, options) =>
 export let formatMessages: typeof types.formatMessages = (messages, options) =>
   ensureServiceIsRunning().formatMessages(messages, options);
 
+export let analyzeMetafile: typeof types.analyzeMetafile = (messages, options) =>
+  ensureServiceIsRunning().analyzeMetafile(messages, options);
+
 export let buildSync: typeof types.buildSync = (options: types.BuildOptions): any => {
   // Try using a long-lived worker thread to avoid repeated start-up overhead
   if (worker_threads && !isInternalWorkerThread) {
@@ -225,6 +197,24 @@ export let formatMessagesSync: typeof types.formatMessagesSync = (messages, opti
   return result!;
 };
 
+export let analyzeMetafileSync: typeof types.analyzeMetafileSync = (metafile, options) => {
+  // Try using a long-lived worker thread to avoid repeated start-up overhead
+  if (worker_threads && !isInternalWorkerThread) {
+    if (!workerThreadService) workerThreadService = startWorkerThreadService(worker_threads);
+    return workerThreadService.analyzeMetafileSync(metafile, options);
+  }
+
+  let result: string;
+  runServiceSync(service => service.analyzeMetafile({
+    callName: 'analyzeMetafileSync',
+    refs: null,
+    metafile: typeof metafile === 'string' ? metafile : JSON.stringify(metafile),
+    options,
+    callback: (err, res) => { if (err) throw err; result = res! },
+  }));
+  return result!;
+};
+
 let initializeWasCalled = false;
 
 export let initialize: typeof types.initialize = options => {
@@ -242,6 +232,7 @@ interface Service {
   serve: typeof types.serve;
   transform: typeof types.transform;
   formatMessages: typeof types.formatMessages;
+  analyzeMetafile: typeof types.analyzeMetafile;
 }
 
 let defaultWD = process.cwd();
@@ -334,6 +325,16 @@ let ensureServiceIsRunning = (): Service => {
           callback: (err, res) => err ? reject(err) : resolve(res!),
         }));
     },
+    analyzeMetafile: (metafile, options) => {
+      return new Promise((resolve, reject) =>
+        service.analyzeMetafile({
+          callName: 'analyzeMetafile',
+          refs,
+          metafile: typeof metafile === 'string' ? metafile : JSON.stringify(metafile),
+          options,
+          callback: (err, res) => err ? reject(err) : resolve(res!),
+        }));
+    },
   };
   return longLivedService;
 }
@@ -380,6 +381,7 @@ interface WorkerThreadService {
   buildSync(options: types.BuildOptions): types.BuildResult;
   transformSync: typeof types.transformSync;
   formatMessagesSync: typeof types.formatMessagesSync;
+  analyzeMetafileSync: typeof types.analyzeMetafileSync;
 }
 
 let workerThreadService: WorkerThreadService | null = null;
@@ -480,6 +482,9 @@ let startWorkerThreadService = (worker_threads: typeof import('worker_threads'))
     formatMessagesSync(messages, options) {
       return runCallSync('formatMessages', [messages, options]);
     },
+    analyzeMetafileSync(metafile, options) {
+      return runCallSync('analyzeMetafile', [metafile, options]);
+    },
   };
 };
 
@@ -512,14 +517,25 @@ let startSyncServiceWorker = () => {
       let sharedBufferView = new Int32Array(sharedBuffer);
 
       try {
-        if (command === 'build') {
-          workerPort.postMessage({ id, resolve: await service.build(args[0]) });
-        } else if (command === 'transform') {
-          workerPort.postMessage({ id, resolve: await service.transform(args[0], args[1]) });
-        } else if (command === 'formatMessages') {
-          workerPort.postMessage({ id, resolve: await service.formatMessages(args[0], args[1]) });
-        } else {
-          throw new Error(`Invalid command: ${command}`);
+        switch (command) {
+          case 'build':
+            workerPort.postMessage({ id, resolve: await service.build(args[0]) });
+            break;
+
+          case 'transform':
+            workerPort.postMessage({ id, resolve: await service.transform(args[0], args[1]) });
+            break;
+
+          case 'formatMessages':
+            workerPort.postMessage({ id, resolve: await service.formatMessages(args[0], args[1]) });
+            break;
+
+          case 'analyzeMetafile':
+            workerPort.postMessage({ id, resolve: await service.analyzeMetafile(args[0], args[1]) });
+            break;
+
+          default:
+            throw new Error(`Invalid command: ${command}`);
         }
       } catch (reject) {
         workerPort.postMessage({ id, reject, properties: extractProperties(reject) });
