@@ -126,6 +126,17 @@ export interface FormatMsgsResponse {
   messages: string[];
 }
 
+export interface AnalyzeMetafileRequest {
+  command: 'analyze-metafile';
+  metafile: string;
+  color?: boolean;
+  verbose?: boolean;
+}
+
+export interface AnalyzeMetafileResponse {
+  result: string;
+}
+
 export interface OnStartRequest {
   command: 'start';
   key: number;
