@@ -105,10 +105,18 @@ const (
 	LegalCommentsExternal
 )
 
+type JSXMode uint8
+
+const (
+	JSXModeTransform JSXMode = iota
+	JSXModePreserve
+)
+
 type Target uint8
 
 const (
-	ESNext Target = iota
+	DefaultTarget Target = iota
+	ESNext
 	ES5
 	ES2015
 	ES2016
@@ -116,6 +124,7 @@ const (
 	ES2018
 	ES2019
 	ES2020
+	ES2021
 )
 
 type Loader uint8
@@ -226,76 +235,79 @@ type TreeShaking uint8
 
 const (
 	TreeShakingDefault TreeShaking = iota
-	TreeShakingIgnoreAnnotations
+	TreeShakingFalse
+	TreeShakingTrue
 )
 
 ////////////////////////////////////////////////////////////////////////////////
 // Build API
 
 type BuildOptions struct {
-	Color    StderrColor
-	LogLimit int
-	LogLevel LogLevel
+	Color    StderrColor // Documentation: https://esbuild.github.io/api/#color
+	LogLimit int         // Documentation: https://esbuild.github.io/api/#log-limit
+	LogLevel LogLevel    // Documentation: https://esbuild.github.io/api/#log-level
 
-	Sourcemap      SourceMap
-	SourceRoot     string
-	SourcesContent SourcesContent
+	Sourcemap      SourceMap      // Documentation: https://esbuild.github.io/api/#sourcemap
+	SourceRoot     string         // Documentation: https://esbuild.github.io/api/#source-root
+	SourcesContent SourcesContent // Documentation: https://esbuild.github.io/api/#sources-content
 
-	Target  Target
-	Engines []Engine
+	Target  Target   // Documentation: https://esbuild.github.io/api/#target
+	Engines []Engine // Documentation: https://esbuild.github.io/api/#target
 
-	MinifyWhitespace  bool
-	MinifyIdentifiers bool
-	MinifySyntax      bool
-	Charset           Charset
-	TreeShaking       TreeShaking
-	LegalComments     LegalComments
+	MinifyWhitespace  bool          // Documentation: https://esbuild.github.io/api/#minify
+	MinifyIdentifiers bool          // Documentation: https://esbuild.github.io/api/#minify
+	MinifySyntax      bool          // Documentation: https://esbuild.github.io/api/#minify
+	Charset           Charset       // Documentation: https://esbuild.github.io/api/#charset
+	TreeShaking       TreeShaking   // Documentation: https://esbuild.github.io/api/#tree-shaking
+	IgnoreAnnotations bool          // Documentation: https://esbuild.github.io/api/#ignore-annotations
+	LegalComments     LegalComments // Documentation: https://esbuild.github.io/api/#legal-comments
 
-	JSXFactory  string
-	JSXFragment string
+	JSXMode     JSXMode // Documentation: https://esbuild.github.io/api/#jsx-mode
+	JSXFactory  string  // Documentation: https://esbuild.github.io/api/#jsx-factory
+	JSXFragment string  // Documentation: https://esbuild.github.io/api/#jsx-fragment
 
-	Define    map[string]string
-	Pure      []string
-	KeepNames bool
+	Define    map[string]string // Documentation: https://esbuild.github.io/api/#define
+	Pure      []string          // Documentation: https://esbuild.github.io/api/#pure
+	KeepNames bool              // Documentation: https://esbuild.github.io/api/#keep-names
 
-	GlobalName        string
-	Bundle            bool
-	PreserveSymlinks  bool
-	Splitting         bool
-	Outfile           string
-	Metafile          bool
-	Outdir            string
-	Outbase           string
-	AbsWorkingDir     string
-	Platform          Platform
-	Format            Format
-	External          []string
-	MainFields        []string
-	Conditions        []string // For the "exports" field in "package.json"
-	Loader            map[string]Loader
-	ResolveExtensions []string
-	Tsconfig          string
-	OutExtensions     map[string]string
-	PublicPath        string
-	Inject            []string
-	Banner            map[string]string
-	Footer            map[string]string
-	NodePaths         []string // The "NODE_PATH" variable from Node.js
+	GlobalName        string            // Documentation: https://esbuild.github.io/api/#global-name
+	Bundle            bool              // Documentation: https://esbuild.github.io/api/#bundle
+	PreserveSymlinks  bool              // Documentation: https://esbuild.github.io/api/#preserve-symlinks
+	Splitting         bool              // Documentation: https://esbuild.github.io/api/#splitting
+	Outfile           string            // Documentation: https://esbuild.github.io/api/#outfile
+	Metafile          bool              // Documentation: https://esbuild.github.io/api/#metafile
+	Outdir            string            // Documentation: https://esbuild.github.io/api/#outdir
+	Outbase           string            // Documentation: https://esbuild.github.io/api/#outbase
+	AbsWorkingDir     string            // Documentation: https://esbuild.github.io/api/#working-directory
+	Platform          Platform          // Documentation: https://esbuild.github.io/api/#platform
+	Format            Format            // Documentation: https://esbuild.github.io/api/#format
+	External          []string          // Documentation: https://esbuild.github.io/api/#external
+	MainFields        []string          // Documentation: https://esbuild.github.io/api/#main-fields
+	Conditions        []string          // Documentation: https://esbuild.github.io/api/#conditions
+	Loader            map[string]Loader // Documentation: https://esbuild.github.io/api/#loader
+	ResolveExtensions []string          // Documentation: https://esbuild.github.io/api/#resolve-extensions
+	Tsconfig          string            // Documentation: https://esbuild.github.io/api/#tsconfig
+	OutExtensions     map[string]string // Documentation: https://esbuild.github.io/api/#out-extension
+	PublicPath        string            // Documentation: https://esbuild.github.io/api/#public-path
+	Inject            []string          // Documentation: https://esbuild.github.io/api/#inject
+	Banner            map[string]string // Documentation: https://esbuild.github.io/api/#banner
+	Footer            map[string]string // Documentation: https://esbuild.github.io/api/#footer
+	NodePaths         []string          // Documentation: https://esbuild.github.io/api/#node-paths
 
-	EntryNames string
-	ChunkNames string
-	AssetNames string
+	EntryNames string // Documentation: https://esbuild.github.io/api/#entry-names
+	ChunkNames string // Documentation: https://esbuild.github.io/api/#chunk-names
+	AssetNames string // Documentation: https://esbuild.github.io/api/#asset-names
 
-	EntryPoints         []string
-	EntryPointsAdvanced []EntryPoint
+	EntryPoints         []string     // Documentation: https://esbuild.github.io/api/#entry-points
+	EntryPointsAdvanced []EntryPoint // Documentation: https://esbuild.github.io/api/#entry-points
 
-	Stdin          *StdinOptions
-	Write          bool
-	AllowOverwrite bool
-	Incremental    bool
-	Plugins        []Plugin
+	Stdin          *StdinOptions // Documentation: https://esbuild.github.io/api/#stdin
+	Write          bool          // Documentation: https://esbuild.github.io/api/#write
+	AllowOverwrite bool          // Documentation: https://esbuild.github.io/api/#allow-overwrite
+	Incremental    bool          // Documentation: https://esbuild.github.io/api/#incremental
+	Plugins        []Plugin      // Documentation: https://esbuild.github.io/plugins/
 
-	Watch *WatchMode
+	Watch *WatchMode // Documentation: https://esbuild.github.io/api/#watch
 }
 
 type EntryPoint struct {
@@ -330,6 +342,7 @@ type OutputFile struct {
 	Contents []byte
 }
 
+// Documentation: https://esbuild.github.io/api/#build-api
 func Build(options BuildOptions) BuildResult {
 	return buildImpl(options).result
 }
@@ -338,38 +351,42 @@ func Build(options BuildOptions) BuildResult {
 // Transform API
 
 type TransformOptions struct {
-	Color    StderrColor
-	LogLimit int
-	LogLevel LogLevel
+	Color    StderrColor // Documentation: https://esbuild.github.io/api/#color
+	LogLimit int         // Documentation: https://esbuild.github.io/api/#log-limit
+	LogLevel LogLevel    // Documentation: https://esbuild.github.io/api/#log-level
 
-	Sourcemap      SourceMap
-	SourceRoot     string
-	SourcesContent SourcesContent
+	Sourcemap      SourceMap      // Documentation: https://esbuild.github.io/api/#sourcemap
+	SourceRoot     string         // Documentation: https://esbuild.github.io/api/#source-root
+	SourcesContent SourcesContent // Documentation: https://esbuild.github.io/api/#sources-content
 
-	Target     Target
-	Format     Format
-	GlobalName string
-	Engines    []Engine
+	Target  Target   // Documentation: https://esbuild.github.io/api/#target
+	Engines []Engine // Documentation: https://esbuild.github.io/api/#target
 
-	MinifyWhitespace  bool
-	MinifyIdentifiers bool
-	MinifySyntax      bool
-	Charset           Charset
-	TreeShaking       TreeShaking
-	LegalComments     LegalComments
+	Format     Format // Documentation: https://esbuild.github.io/api/#format
+	GlobalName string // Documentation: https://esbuild.github.io/api/#global-name
 
-	JSXFactory  string
-	JSXFragment string
-	TsconfigRaw string
-	Footer      string
-	Banner      string
+	MinifyWhitespace  bool          // Documentation: https://esbuild.github.io/api/#minify
+	MinifyIdentifiers bool          // Documentation: https://esbuild.github.io/api/#minify
+	MinifySyntax      bool          // Documentation: https://esbuild.github.io/api/#minify
+	Charset           Charset       // Documentation: https://esbuild.github.io/api/#charset
+	TreeShaking       TreeShaking   // Documentation: https://esbuild.github.io/api/#tree-shaking
+	IgnoreAnnotations bool          // Documentation: https://esbuild.github.io/api/#ignore-annotations
+	LegalComments     LegalComments // Documentation: https://esbuild.github.io/api/#legal-comments
 
-	Define    map[string]string
-	Pure      []string
-	KeepNames bool
+	JSXMode     JSXMode // Documentation: https://esbuild.github.io/api/#jsx
+	JSXFactory  string  // Documentation: https://esbuild.github.io/api/#jsx-factory
+	JSXFragment string  // Documentation: https://esbuild.github.io/api/#jsx-fragment
 
-	Sourcefile string
-	Loader     Loader
+	TsconfigRaw string // Documentation: https://esbuild.github.io/api/#tsconfig-raw
+	Banner      string // Documentation: https://esbuild.github.io/api/#banner
+	Footer      string // Documentation: https://esbuild.github.io/api/#footer
+
+	Define    map[string]string // Documentation: https://esbuild.github.io/api/#define
+	Pure      []string          // Documentation: https://esbuild.github.io/api/#pure
+	KeepNames bool              // Documentation: https://esbuild.github.io/api/#keep-names
+
+	Sourcefile string // Documentation: https://esbuild.github.io/api/#sourcefile
+	Loader     Loader // Documentation: https://esbuild.github.io/api/#loader
 }
 
 type TransformResult struct {
@@ -380,6 +397,7 @@ type TransformResult struct {
 	Map  []byte
 }
 
+// Documentation: https://esbuild.github.io/api/#transform-api
 func Transform(input string, options TransformOptions) TransformResult {
 	return transformImpl(input, options)
 }
@@ -387,6 +405,7 @@ func Transform(input string, options TransformOptions) TransformResult {
 ////////////////////////////////////////////////////////////////////////////////
 // Serve API
 
+// Documentation: https://esbuild.github.io/api/#serve-arguments
 type ServeOptions struct {
 	Port      uint16
 	Host      string
@@ -402,6 +421,7 @@ type ServeOnRequestArgs struct {
 	TimeInMS      int // The time to generate the response, not to send it
 }
 
+// Documentation: https://esbuild.github.io/api/#serve-return-values
 type ServeResult struct {
 	Port uint16
 	Host string
@@ -409,6 +429,7 @@ type ServeResult struct {
 	Stop func()
 }
 
+// Documentation: https://esbuild.github.io/api/#serve
 func Serve(serveOptions ServeOptions, buildOptions BuildOptions) (ServeResult, error) {
 	return serveImpl(serveOptions, buildOptions)
 }
@@ -416,6 +437,13 @@ func Serve(serveOptions ServeOptions, buildOptions BuildOptions) (ServeResult, e
 ////////////////////////////////////////////////////////////////////////////////
 // Plugin API
 
+type SideEffects uint8
+
+const (
+	SideEffectsTrue SideEffects = iota
+	SideEffectsFalse
+)
+
 type Plugin struct {
 	Name  string
 	Setup func(PluginBuild)
@@ -454,10 +482,11 @@ type OnResolveResult struct {
 	Errors   []Message
 	Warnings []Message
 
-	Path       string
-	External   bool
-	Namespace  string
-	PluginData interface{}
+	Path        string
+	External    bool
+	SideEffects SideEffects
+	Namespace   string
+	PluginData  interface{}
 
 	WatchFiles []string
 	WatchDirs  []string
@@ -520,3 +549,16 @@ type FormatMessagesOptions struct {
 func FormatMessages(msgs []Message, opts FormatMessagesOptions) []string {
 	return formatMsgsImpl(msgs, opts)
 }
+
+////////////////////////////////////////////////////////////////////////////////
+// AnalyzeMetafile API
+
+type AnalyzeMetafileOptions struct {
+	Color   bool
+	Verbose bool
+}
+
+// Documentation: https://esbuild.github.io/api/#analyze
+func AnalyzeMetafile(metafile string, opts AnalyzeMetafileOptions) string {
+	return analyzeMetafileImpl(metafile, opts)
+}
