@@ -3,6 +3,7 @@ package api
 import (
 	"fmt"
 	"io/ioutil"
+	"math"
 	"math/rand"
 	"os"
 	"regexp"
@@ -12,6 +13,7 @@ import (
 	"sync"
 	"sync/atomic"
 	"time"
+	"unicode/utf8"
 
 	"github.com/evanw/esbuild/internal/api_helpers"
 	"github.com/evanw/esbuild/internal/ast"
@@ -63,6 +65,10 @@ func validatePathTemplate(template string) []config.PathTemplate {
 			placeholder = config.HashPlaceholder
 			search += len("[hash]")
 
+		case strings.HasPrefix(tail, "[ext]"):
+			placeholder = config.ExtPlaceholder
+			search += len("[ext]")
+
 		default:
 			// Skip past the "[" so we don't find it again
 			search++
@@ -202,11 +208,19 @@ func validateASCIIOnly(value Charset) bool {
 	}
 }
 
-func validateIgnoreDCEAnnotations(value TreeShaking) bool {
+func validateTreeShaking(value TreeShaking, bundle bool, format Format) bool {
 	switch value {
 	case TreeShakingDefault:
+		// If we're in an IIFE then there's no way to concatenate additional code
+		// to the end of our output so we assume tree shaking is safe. And when
+		// bundling we assume that tree shaking is safe because if you want to add
+		// code to the bundle, you should be doing that by including it in the
+		// bundle instead of concatenating it afterward, so we also assume tree
+		// shaking is safe then. Otherwise we assume tree shaking is not safe.
+		return bundle || format == FormatIIFE
+	case TreeShakingFalse:
 		return false
-	case TreeShakingIgnoreAnnotations:
+	case TreeShakingTrue:
 		return true
 	default:
 		panic("Invalid tree shaking")
@@ -267,9 +281,14 @@ func validateEngine(value EngineName) compat.Engine {
 
 var versionRegex = regexp.MustCompile(`^([0-9]+)(?:\.([0-9]+))?(?:\.([0-9]+))?$`)
 
-func validateFeatures(log logger.Log, target Target, engines []Engine) (compat.JSFeature, compat.CSSFeature, string) {
+func validateFeatures(log logger.Log, target Target, engines []Engine) (config.TargetFromAPI, compat.JSFeature, compat.CSSFeature, string) {
+	if target == DefaultTarget && len(engines) == 0 {
+		return config.TargetWasUnconfigured, 0, 0, ""
+	}
+
 	constraints := make(map[compat.Engine][]int)
 	targets := make([]string, 0, 1+len(engines))
+	targetFromAPI := config.TargetWasConfigured
 
 	switch target {
 	case ES5:
@@ -286,7 +305,11 @@ func validateFeatures(log logger.Log, target Target, engines []Engine) (compat.J
 		constraints[compat.ES] = []int{2019}
 	case ES2020:
 		constraints[compat.ES] = []int{2020}
+	case ES2021:
+		constraints[compat.ES] = []int{2021}
 	case ESNext:
+		targetFromAPI = config.TargetWasConfiguredIncludingESNext
+	case DefaultTarget:
 	default:
 		panic("Invalid target")
 	}
@@ -321,7 +344,7 @@ func validateFeatures(log logger.Log, target Target, engines []Engine) (compat.J
 			}
 		}
 
-		log.AddError(nil, logger.Loc{}, fmt.Sprintf("Invalid version: %q", engine.Version))
+		log.Add(logger.Error, nil, logger.Range{}, fmt.Sprintf("Invalid version: %q", engine.Version))
 	}
 
 	for engine, version := range constraints {
@@ -340,7 +363,7 @@ func validateFeatures(log logger.Log, target Target, engines []Engine) (compat.J
 	sort.Strings(targets)
 	targetEnv := strings.Join(targets, ", ")
 
-	return compat.UnsupportedJSFeatures(constraints), compat.UnsupportedCSSFeatures(constraints), targetEnv
+	return targetFromAPI, compat.UnsupportedJSFeatures(constraints), compat.UnsupportedCSSFeatures(constraints), targetEnv
 }
 
 func validateGlobalName(log logger.Log, text string) []string {
@@ -367,7 +390,7 @@ func validateExternals(log logger.Log, fs fs.FS, paths []string) config.External
 	for _, path := range paths {
 		if index := strings.IndexByte(path, '*'); index != -1 {
 			if strings.ContainsRune(path[index+1:], '*') {
-				log.AddError(nil, logger.Loc{}, fmt.Sprintf("External path %q cannot have more than one \"*\" wildcard", path))
+				log.Add(logger.Error, nil, logger.Range{}, fmt.Sprintf("External path %q cannot have more than one \"*\" wildcard", path))
 			} else {
 				result.Patterns = append(result.Patterns, config.WildcardPattern{
 					Prefix: path[:index],
@@ -393,7 +416,7 @@ func validateResolveExtensions(log logger.Log, order []string) []string {
 	}
 	for _, ext := range order {
 		if !isValidExtension(ext) {
-			log.AddError(nil, logger.Loc{}, fmt.Sprintf("Invalid file extension: %q", ext))
+			log.Add(logger.Error, nil, logger.Range{}, fmt.Sprintf("Invalid file extension: %q", ext))
 		}
 	}
 	return order
@@ -404,7 +427,7 @@ func validateLoaders(log logger.Log, loaders map[string]Loader) map[string]confi
 	if loaders != nil {
 		for ext, loader := range loaders {
 			if !isValidExtension(ext) {
-				log.AddError(nil, logger.Loc{}, fmt.Sprintf("Invalid file extension: %q", ext))
+				log.Add(logger.Error, nil, logger.Range{}, fmt.Sprintf("Invalid file extension: %q", ext))
 			}
 			result[ext] = validateLoader(loader)
 		}
@@ -412,18 +435,12 @@ func validateLoaders(log logger.Log, loaders map[string]Loader) map[string]confi
 	return result
 }
 
-func validateJSX(log logger.Log, text string, name string) []string {
-	if text == "" {
-		return nil
+func validateJSXExpr(log logger.Log, text string, name string, kind js_parser.JSXExprKind) config.JSXExpr {
+	if expr, ok := js_parser.ParseJSXExpr(text, kind); ok {
+		return expr
 	}
-	parts := strings.Split(text, ".")
-	for _, part := range parts {
-		if !js_lexer.IsIdentifier(part) {
-			log.AddError(nil, logger.Loc{}, fmt.Sprintf("Invalid JSX %s: %q", name, text))
-			return nil
-		}
-	}
-	return parts
+	log.Add(logger.Error, nil, logger.Range{}, fmt.Sprintf("Invalid JSX %s: %q", name, text))
+	return config.JSXExpr{}
 }
 
 func validateDefines(
@@ -442,9 +459,9 @@ func validateDefines(
 		for _, part := range strings.Split(key, ".") {
 			if !js_lexer.IsIdentifier(part) {
 				if part == key {
-					log.AddError(nil, logger.Loc{}, fmt.Sprintf("The define key %q must be a valid identifier", key))
+					log.Add(logger.Error, nil, logger.Range{}, fmt.Sprintf("The define key %q must be a valid identifier", key))
 				} else {
-					log.AddError(nil, logger.Loc{}, fmt.Sprintf("The define key %q contains invalid identifier %q", key, part))
+					log.Add(logger.Error, nil, logger.Range{}, fmt.Sprintf("The define key %q contains invalid identifier %q", key, part))
 				}
 				continue
 			}
@@ -465,9 +482,9 @@ func validateDefines(
 
 		// Parse the value as JSON
 		source := logger.Source{Contents: value}
-		expr, ok := js_parser.ParseJSON(logger.NewDeferLog(), source, js_parser.JSONOptions{})
+		expr, ok := js_parser.ParseJSON(logger.NewDeferLog(logger.DeferLogAll), source, js_parser.JSONOptions{})
 		if !ok {
-			log.AddError(nil, logger.Loc{}, fmt.Sprintf("Invalid define value (must be valid JSON syntax or a single identifier): %s", value))
+			log.Add(logger.Error, nil, logger.Range{}, fmt.Sprintf("Invalid define value (must be valid JSON syntax or a single identifier): %s", value))
 			continue
 		}
 
@@ -475,7 +492,7 @@ func validateDefines(
 		switch e := expr.Data.(type) {
 		// These values are inserted inline, and can participate in constant folding
 		case *js_ast.ENull:
-			fn = func(config.DefineArgs) js_ast.E { return &js_ast.ENull{} }
+			fn = func(config.DefineArgs) js_ast.E { return js_ast.ENullShared }
 		case *js_ast.EBoolean:
 			fn = func(config.DefineArgs) js_ast.E { return &js_ast.EBoolean{Value: e.Value} }
 		case *js_ast.EString:
@@ -541,7 +558,7 @@ func validateDefines(
 		// The key must be a dot-separated identifier list
 		for _, part := range strings.Split(key, ".") {
 			if !js_lexer.IsIdentifier(part) {
-				log.AddError(nil, logger.Loc{}, fmt.Sprintf("Invalid pure function: %q", key))
+				log.Add(logger.Error, nil, logger.Range{}, fmt.Sprintf("Invalid pure function: %q", key))
 				continue
 			}
 		}
@@ -564,7 +581,7 @@ func validatePath(log logger.Log, fs fs.FS, relPath string, pathKind string) str
 	}
 	absPath, ok := fs.Abs(relPath)
 	if !ok {
-		log.AddError(nil, logger.Loc{}, fmt.Sprintf("Invalid %s: %s", pathKind, relPath))
+		log.Add(logger.Error, nil, logger.Range{}, fmt.Sprintf("Invalid %s: %s", pathKind, relPath))
 	}
 	return absPath
 }
@@ -572,7 +589,7 @@ func validatePath(log logger.Log, fs fs.FS, relPath string, pathKind string) str
 func validateOutputExtensions(log logger.Log, outExtensions map[string]string) (js string, css string) {
 	for key, value := range outExtensions {
 		if !isValidExtension(value) {
-			log.AddError(nil, logger.Loc{}, fmt.Sprintf("Invalid output extension: %q", value))
+			log.Add(logger.Error, nil, logger.Range{}, fmt.Sprintf("Invalid output extension: %q", value))
 		}
 		switch key {
 		case ".js":
@@ -580,7 +597,7 @@ func validateOutputExtensions(log logger.Log, outExtensions map[string]string) (
 		case ".css":
 			css = value
 		default:
-			log.AddError(nil, logger.Loc{}, fmt.Sprintf("Invalid output extension: %q (valid: .css, .js)", key))
+			log.Add(logger.Error, nil, logger.Range{}, fmt.Sprintf("Invalid output extension: %q (valid: .css, .js)", key))
 		}
 	}
 	return
@@ -594,7 +611,7 @@ func validateBannerOrFooter(log logger.Log, name string, values map[string]strin
 		case "css":
 			css = value
 		default:
-			log.AddError(nil, logger.Loc{}, fmt.Sprintf("Invalid %s file type: %q (valid: css, js)", name, key))
+			log.Add(logger.Error, nil, logger.Range{}, fmt.Sprintf("Invalid %s file type: %q (valid: css, js)", name, key))
 		}
 	}
 	return
@@ -704,7 +721,7 @@ func buildImpl(buildOpts BuildOptions) internalBuildResult {
 		AbsWorkingDir: buildOpts.AbsWorkingDir,
 	})
 	if err != nil {
-		log.AddError(nil, logger.Loc{}, err.Error())
+		log.Add(logger.Error, nil, logger.Range{}, err.Error())
 		return internalBuildResult{result: BuildResult{Errors: convertMessagesToPublic(logger.Error, log.Done())}}
 	}
 
@@ -729,6 +746,20 @@ func buildImpl(buildOpts BuildOptions) internalBuildResult {
 	return internalResult
 }
 
+func prettyPrintByteCount(n int) string {
+	var size string
+	if n < 1024 {
+		size = fmt.Sprintf("%db ", n)
+	} else if n < 1024*1024 {
+		size = fmt.Sprintf("%.1fkb", float64(n)/(1024))
+	} else if n < 1024*1024*1024 {
+		size = fmt.Sprintf("%.1fmb", float64(n)/(1024*1024))
+	} else {
+		size = fmt.Sprintf("%.1fgb", float64(n)/(1024*1024*1024))
+	}
+	return size
+}
+
 func printSummary(logOptions logger.OutputOptions, outputFiles []OutputFile, start time.Time) {
 	var table logger.SummaryTable = make([]logger.SummaryTableEntry, len(outputFiles))
 
@@ -742,20 +773,10 @@ func printSummary(logOptions logger.OutputOptions, outputFiles []OutputFile, sta
 					}
 					base := realFS.Base(path)
 					n := len(file.Contents)
-					var size string
-					if n < 1024 {
-						size = fmt.Sprintf("%db ", n)
-					} else if n < 1024*1024 {
-						size = fmt.Sprintf("%.1fkb", float64(n)/(1024))
-					} else if n < 1024*1024*1024 {
-						size = fmt.Sprintf("%.1fmb", float64(n)/(1024*1024))
-					} else {
-						size = fmt.Sprintf("%.1fgb", float64(n)/(1024*1024*1024))
-					}
 					table[i] = logger.SummaryTableEntry{
 						Dir:         path[:len(path)-len(base)],
 						Base:        base,
-						Size:        size,
+						Size:        prettyPrintByteCount(n),
 						Bytes:       n,
 						IsSourceMap: strings.HasSuffix(base, ".map"),
 					}
@@ -794,19 +815,21 @@ func rebuildImpl(
 		// This should already have been checked above
 		panic(err.Error())
 	}
-	jsFeatures, cssFeatures, targetEnv := validateFeatures(log, buildOpts.Target, buildOpts.Engines)
+	targetFromAPI, jsFeatures, cssFeatures, targetEnv := validateFeatures(log, buildOpts.Target, buildOpts.Engines)
 	outJS, outCSS := validateOutputExtensions(log, buildOpts.OutExtensions)
 	bannerJS, bannerCSS := validateBannerOrFooter(log, "banner", buildOpts.Banner)
 	footerJS, footerCSS := validateBannerOrFooter(log, "footer", buildOpts.Footer)
 	minify := buildOpts.MinifyWhitespace && buildOpts.MinifyIdentifiers && buildOpts.MinifySyntax
 	defines, injectedDefines := validateDefines(log, buildOpts.Define, buildOpts.Pure, buildOpts.Platform, minify)
 	options := config.Options{
+		TargetFromAPI:          targetFromAPI,
 		UnsupportedJSFeatures:  jsFeatures,
 		UnsupportedCSSFeatures: cssFeatures,
 		OriginalTargetEnv:      targetEnv,
 		JSX: config.JSXOptions{
-			Factory:  validateJSX(log, buildOpts.JSXFactory, "factory"),
-			Fragment: validateJSX(log, buildOpts.JSXFragment, "fragment"),
+			Preserve: buildOpts.JSXMode == JSXModePreserve,
+			Factory:  validateJSXExpr(log, buildOpts.JSXFactory, "factory", js_parser.JSXFactory),
+			Fragment: validateJSXExpr(log, buildOpts.JSXFragment, "fragment", js_parser.JSXFragment),
 		},
 		Defines:               defines,
 		InjectedDefines:       injectedDefines,
@@ -820,7 +843,8 @@ func rebuildImpl(
 		MinifyIdentifiers:     buildOpts.MinifyIdentifiers,
 		AllowOverwrite:        buildOpts.AllowOverwrite,
 		ASCIIOnly:             validateASCIIOnly(buildOpts.Charset),
-		IgnoreDCEAnnotations:  validateIgnoreDCEAnnotations(buildOpts.TreeShaking),
+		IgnoreDCEAnnotations:  buildOpts.IgnoreAnnotations,
+		TreeShaking:           validateTreeShaking(buildOpts.TreeShaking, buildOpts.Bundle, buildOpts.Format),
 		GlobalName:            validateGlobalName(log, buildOpts.GlobalName),
 		CodeSplitting:         buildOpts.Splitting,
 		OutputFormat:          validateFormat(buildOpts.Format),
@@ -879,13 +903,13 @@ func rebuildImpl(
 	}
 
 	if options.AbsOutputDir == "" && entryPointCount > 1 {
-		log.AddError(nil, logger.Loc{},
+		log.Add(logger.Error, nil, logger.Range{},
 			"Must use \"outdir\" when there are multiple input files")
 	} else if options.AbsOutputDir == "" && options.CodeSplitting {
-		log.AddError(nil, logger.Loc{},
+		log.Add(logger.Error, nil, logger.Range{},
 			"Must use \"outdir\" when code splitting is enabled")
 	} else if options.AbsOutputFile != "" && options.AbsOutputDir != "" {
-		log.AddError(nil, logger.Loc{}, "Cannot use both \"outfile\" and \"outdir\"")
+		log.Add(logger.Error, nil, logger.Range{}, "Cannot use both \"outfile\" and \"outdir\"")
 	} else if options.AbsOutputFile != "" {
 		// If the output file is specified, use it to derive the output directory
 		options.AbsOutputDir = realFS.Dir(options.AbsOutputFile)
@@ -894,14 +918,14 @@ func rebuildImpl(
 
 		// Forbid certain features when writing to stdout
 		if options.SourceMap != config.SourceMapNone && options.SourceMap != config.SourceMapInline {
-			log.AddError(nil, logger.Loc{}, "Cannot use an external source map without an output path")
+			log.Add(logger.Error, nil, logger.Range{}, "Cannot use an external source map without an output path")
 		}
 		if options.LegalComments.HasExternalFile() {
-			log.AddError(nil, logger.Loc{}, "Cannot use linked or external legal comments without an output path")
+			log.Add(logger.Error, nil, logger.Range{}, "Cannot use linked or external legal comments without an output path")
 		}
 		for _, loader := range options.ExtensionToLoader {
 			if loader == config.LoaderFile {
-				log.AddError(nil, logger.Loc{}, "Cannot use the \"file\" loader without an output path")
+				log.Add(logger.Error, nil, logger.Range{}, "Cannot use the \"file\" loader without an output path")
 				break
 			}
 		}
@@ -914,7 +938,7 @@ func rebuildImpl(
 	if !buildOpts.Bundle {
 		// Disallow bundle-only options when not bundling
 		if len(options.ExternalModules.NodeModules) > 0 || len(options.ExternalModules.AbsPaths) > 0 {
-			log.AddError(nil, logger.Loc{}, "Cannot use \"external\" without \"bundle\"")
+			log.Add(logger.Error, nil, logger.Range{}, "Cannot use \"external\" without \"bundle\"")
 		}
 	} else if options.OutputFormat == config.FormatPreserve {
 		// If the format isn't specified, set the default format using the platform
@@ -937,7 +961,7 @@ func rebuildImpl(
 
 	// Code splitting is experimental and currently only enabled for ES6 modules
 	if options.CodeSplitting && options.OutputFormat != config.FormatESModule {
-		log.AddError(nil, logger.Loc{}, "Splitting currently only works with the \"esm\" format")
+		log.Add(logger.Error, nil, logger.Range{}, "Splitting currently only works with the \"esm\" format")
 	}
 
 	var outputFiles []OutputFile
@@ -960,10 +984,11 @@ func rebuildImpl(
 		if !log.HasErrors() {
 			// Compile the bundle
 			results, metafile := bundle.Compile(log, options, timer)
-			metafileJSON = metafile
 
 			// Stop now if there were errors
 			if !log.HasErrors() {
+				metafileJSON = metafile
+
 				// Flush any deferred warnings now
 				log.AlmostDone()
 
@@ -972,10 +997,10 @@ func rebuildImpl(
 					if options.WriteToStdout {
 						// Special-case writing to stdout
 						if len(results) != 1 {
-							log.AddError(nil, logger.Loc{}, fmt.Sprintf(
+							log.Add(logger.Error, nil, logger.Range{}, fmt.Sprintf(
 								"Internal error: did not expect to generate %d files when writing to stdout", len(results)))
 						} else if _, err := os.Stdout.Write(results[0].Contents); err != nil {
-							log.AddError(nil, logger.Loc{}, fmt.Sprintf(
+							log.Add(logger.Error, nil, logger.Range{}, fmt.Sprintf(
 								"Failed to write to stdout: %s", err.Error()))
 						}
 					} else {
@@ -986,8 +1011,8 @@ func rebuildImpl(
 							go func(result graph.OutputFile) {
 								fs.BeforeFileOpen()
 								defer fs.AfterFileClose()
-								if err := os.MkdirAll(realFS.Dir(result.AbsPath), 0755); err != nil {
-									log.AddError(nil, logger.Loc{}, fmt.Sprintf(
+								if err := fs.MkdirAll(realFS, realFS.Dir(result.AbsPath), 0755); err != nil {
+									log.Add(logger.Error, nil, logger.Range{}, fmt.Sprintf(
 										"Failed to create output directory: %s", err.Error()))
 								} else {
 									var mode os.FileMode = 0644
@@ -995,7 +1020,7 @@ func rebuildImpl(
 										mode = 0755
 									}
 									if err := ioutil.WriteFile(result.AbsPath, result.Contents, mode); err != nil {
-										log.AddError(nil, logger.Loc{}, fmt.Sprintf(
+										log.Add(logger.Error, nil, logger.Range{}, fmt.Sprintf(
 											"Failed to write to output file: %s", err.Error()))
 									}
 								}
@@ -1194,11 +1219,11 @@ func (w *watcher) tryToFindDirtyPath() string {
 
 	// Always check all recent items every iteration
 	for i, path := range w.recentItems {
-		if w.data.Paths[path]() {
+		if dirtyPath := w.data.Paths[path](); dirtyPath != "" {
 			// Move this path to the back of the list (i.e. the "most recent" position)
 			copy(w.recentItems[i:], w.recentItems[i+1:])
 			w.recentItems[len(w.recentItems)-1] = path
-			return path
+			return dirtyPath
 		}
 	}
 
@@ -1212,7 +1237,7 @@ func (w *watcher) tryToFindDirtyPath() string {
 
 	// Check if any of the entries in this iteration have been modified
 	for _, path := range toCheck {
-		if w.data.Paths[path]() {
+		if dirtyPath := w.data.Paths[path](); dirtyPath != "" {
 			// Mark this item as recent by adding it to the back of the list
 			w.recentItems = append(w.recentItems, path)
 			if len(w.recentItems) > maxRecentItemCount {
@@ -1220,7 +1245,7 @@ func (w *watcher) tryToFindDirtyPath() string {
 				copy(w.recentItems, w.recentItems[1:])
 				w.recentItems = w.recentItems[:maxRecentItemCount]
 			}
-			return path
+			return dirtyPath
 		}
 	}
 	return ""
@@ -1238,14 +1263,16 @@ func transformImpl(input string, transformOpts TransformOptions) TransformResult
 	})
 
 	// Settings from the user come first
-	preserveUnusedImportsTS := false
+	unusedImportsTS := config.UnusedImportsRemoveStmt
 	useDefineForClassFieldsTS := config.Unspecified
 	jsx := config.JSXOptions{
-		Factory:  validateJSX(log, transformOpts.JSXFactory, "factory"),
-		Fragment: validateJSX(log, transformOpts.JSXFragment, "fragment"),
+		Preserve: transformOpts.JSXMode == JSXModePreserve,
+		Factory:  validateJSXExpr(log, transformOpts.JSXFactory, "factory", js_parser.JSXFactory),
+		Fragment: validateJSXExpr(log, transformOpts.JSXFragment, "fragment", js_parser.JSXFragment),
 	}
 
 	// Settings from "tsconfig.json" override those
+	var tsTarget *config.TSTarget
 	caches := cache.MakeCacheSet()
 	if transformOpts.TsconfigRaw != "" {
 		source := logger.Source{
@@ -1255,17 +1282,19 @@ func transformImpl(input string, transformOpts TransformOptions) TransformResult
 		}
 		if result := resolver.ParseTSConfigJSON(log, source, &caches.JSONCache, nil); result != nil {
 			if len(result.JSXFactory) > 0 {
-				jsx.Factory = result.JSXFactory
+				jsx.Factory = config.JSXExpr{Parts: result.JSXFactory}
 			}
 			if len(result.JSXFragmentFactory) > 0 {
-				jsx.Fragment = result.JSXFragmentFactory
+				jsx.Fragment = config.JSXExpr{Parts: result.JSXFragmentFactory}
 			}
 			if result.UseDefineForClassFields != config.Unspecified {
 				useDefineForClassFieldsTS = result.UseDefineForClassFields
 			}
-			if result.PreserveImportsNotUsedAsValues {
-				preserveUnusedImportsTS = true
-			}
+			unusedImportsTS = config.UnusedImportsFromTsconfigValues(
+				result.PreserveImportsNotUsedAsValues,
+				result.PreserveValueImports,
+			)
+			tsTarget = result.TSTarget
 		}
 	}
 
@@ -1278,12 +1307,14 @@ func transformImpl(input string, transformOpts TransformOptions) TransformResult
 	}
 
 	// Convert and validate the transformOpts
-	jsFeatures, cssFeatures, targetEnv := validateFeatures(log, transformOpts.Target, transformOpts.Engines)
+	targetFromAPI, jsFeatures, cssFeatures, targetEnv := validateFeatures(log, transformOpts.Target, transformOpts.Engines)
 	defines, injectedDefines := validateDefines(log, transformOpts.Define, transformOpts.Pure, PlatformNeutral, false /* minify */)
 	options := config.Options{
+		TargetFromAPI:           targetFromAPI,
 		UnsupportedJSFeatures:   jsFeatures,
 		UnsupportedCSSFeatures:  cssFeatures,
 		OriginalTargetEnv:       targetEnv,
+		TSTarget:                tsTarget,
 		JSX:                     jsx,
 		Defines:                 defines,
 		InjectedDefines:         injectedDefines,
@@ -1297,11 +1328,12 @@ func transformImpl(input string, transformOpts TransformOptions) TransformResult
 		RemoveWhitespace:        transformOpts.MinifyWhitespace,
 		MinifyIdentifiers:       transformOpts.MinifyIdentifiers,
 		ASCIIOnly:               validateASCIIOnly(transformOpts.Charset),
-		IgnoreDCEAnnotations:    validateIgnoreDCEAnnotations(transformOpts.TreeShaking),
+		IgnoreDCEAnnotations:    transformOpts.IgnoreAnnotations,
+		TreeShaking:             validateTreeShaking(transformOpts.TreeShaking, false /* bundle */, transformOpts.Format),
 		AbsOutputFile:           transformOpts.Sourcefile + "-out",
 		KeepNames:               transformOpts.KeepNames,
 		UseDefineForClassFields: useDefineForClassFieldsTS,
-		PreserveUnusedImportsTS: preserveUnusedImportsTS,
+		UnusedImportsTS:         unusedImportsTS,
 		Stdin: &config.StdinInfo{
 			Loader:     validateLoader(transformOpts.Loader),
 			Contents:   input,
@@ -1317,14 +1349,14 @@ func transformImpl(input string, transformOpts TransformOptions) TransformResult
 	}
 	if options.SourceMap == config.SourceMapLinkedWithComment {
 		// Linked source maps don't make sense because there's no output file name
-		log.AddError(nil, logger.Loc{}, "Cannot transform with linked source maps")
+		log.Add(logger.Error, nil, logger.Range{}, "Cannot transform with linked source maps")
 	}
 	if options.SourceMap != config.SourceMapNone && options.Stdin.SourceFile == "" {
-		log.AddError(nil, logger.Loc{},
+		log.Add(logger.Error, nil, logger.Range{},
 			"Must use \"sourcefile\" with \"sourcemap\" to set the original file name")
 	}
 	if options.LegalComments.HasExternalFile() {
-		log.AddError(nil, logger.Loc{}, "Cannot transform with linked or external legal comments")
+		log.Add(logger.Error, nil, logger.Range{}, "Cannot transform with linked or external legal comments")
 	}
 
 	// Set the output mode using other settings
@@ -1416,7 +1448,7 @@ func (impl *pluginImpl) OnStart(callback func() (OnStartResult, error)) {
 func (impl *pluginImpl) OnResolve(options OnResolveOptions, callback func(OnResolveArgs) (OnResolveResult, error)) {
 	filter, err := config.CompileFilterForPlugin(impl.plugin.Name, "OnResolve", options.Filter)
 	if filter == nil {
-		impl.log.AddError(nil, logger.Loc{}, err.Error())
+		impl.log.Add(logger.Error, nil, logger.Range{}, err.Error())
 		return
 	}
 
@@ -1464,6 +1496,7 @@ func (impl *pluginImpl) OnResolve(options OnResolveOptions, callback func(OnReso
 
 			result.Path = logger.Path{Text: response.Path, Namespace: response.Namespace}
 			result.External = response.External
+			result.IsSideEffectFree = response.SideEffects == SideEffectsFalse
 			result.PluginData = response.PluginData
 
 			// Convert log messages
@@ -1482,7 +1515,7 @@ func (impl *pluginImpl) OnResolve(options OnResolveOptions, callback func(OnReso
 func (impl *pluginImpl) OnLoad(options OnLoadOptions, callback func(OnLoadArgs) (OnLoadResult, error)) {
 	filter, err := config.CompileFilterForPlugin(impl.plugin.Name, "OnLoad", options.Filter)
 	if filter == nil {
-		impl.log.AddError(nil, logger.Loc{}, err.Error())
+		impl.log.Add(logger.Error, nil, logger.Range{}, err.Error())
 		return
 	}
 
@@ -1547,7 +1580,7 @@ func loadPlugins(initialOptions *BuildOptions, fs fs.FS, log logger.Log) (plugin
 
 	for i, item := range clone {
 		if item.Name == "" {
-			log.AddError(nil, logger.Loc{}, fmt.Sprintf("Plugin at index %d is missing a name", i))
+			log.Add(logger.Error, nil, logger.Range{}, fmt.Sprintf("Plugin at index %d is missing a name", i))
 			continue
 		}
 
@@ -1593,3 +1626,314 @@ func formatMsgsImpl(msgs []Message, opts FormatMessagesOptions) []string {
 	}
 	return strings
 }
+
+////////////////////////////////////////////////////////////////////////////////
+// AnalyzeMetafile API
+
+type metafileEntry struct {
+	name       string
+	entryPoint string
+	entries    []metafileEntry
+	size       int
+}
+
+type metafileArray []metafileEntry
+
+func (a metafileArray) Len() int          { return len(a) }
+func (a metafileArray) Swap(i int, j int) { a[i], a[j] = a[j], a[i] }
+
+func (a metafileArray) Less(i int, j int) bool {
+	ai := a[i]
+	aj := a[j]
+	return ai.size > aj.size || (ai.size == aj.size && ai.name < aj.name)
+}
+
+func getObjectProperty(expr js_ast.Expr, key string) js_ast.Expr {
+	if obj, ok := expr.Data.(*js_ast.EObject); ok {
+		for _, prop := range obj.Properties {
+			if js_lexer.UTF16EqualsString(prop.Key.Data.(*js_ast.EString).Value, key) {
+				return prop.ValueOrNil
+			}
+		}
+	}
+	return js_ast.Expr{}
+}
+
+func getObjectPropertyNumber(expr js_ast.Expr, key string) *js_ast.ENumber {
+	value, _ := getObjectProperty(expr, key).Data.(*js_ast.ENumber)
+	return value
+}
+
+func getObjectPropertyString(expr js_ast.Expr, key string) *js_ast.EString {
+	value, _ := getObjectProperty(expr, key).Data.(*js_ast.EString)
+	return value
+}
+
+func getObjectPropertyObject(expr js_ast.Expr, key string) *js_ast.EObject {
+	value, _ := getObjectProperty(expr, key).Data.(*js_ast.EObject)
+	return value
+}
+
+func getObjectPropertyArray(expr js_ast.Expr, key string) *js_ast.EArray {
+	value, _ := getObjectProperty(expr, key).Data.(*js_ast.EArray)
+	return value
+}
+
+func analyzeMetafileImpl(metafile string, opts AnalyzeMetafileOptions) string {
+	log := logger.NewDeferLog(logger.DeferLogNoVerboseOrDebug)
+	source := logger.Source{Contents: metafile}
+
+	if result, ok := js_parser.ParseJSON(log, source, js_parser.JSONOptions{}); ok {
+		if outputs := getObjectPropertyObject(result, "outputs"); outputs != nil {
+			var entries metafileArray
+			var entryPoints []string
+
+			// Scan over the "outputs" object
+			for _, output := range outputs.Properties {
+				if key := js_lexer.UTF16ToString(output.Key.Data.(*js_ast.EString).Value); !strings.HasSuffix(key, ".map") {
+					entryPointPath := ""
+					if entryPoint := getObjectPropertyString(output.ValueOrNil, "entryPoint"); entryPoint != nil {
+						entryPointPath = js_lexer.UTF16ToString(entryPoint.Value)
+						entryPoints = append(entryPoints, entryPointPath)
+					}
+
+					if bytes := getObjectPropertyNumber(output.ValueOrNil, "bytes"); bytes != nil {
+						if inputs := getObjectPropertyObject(output.ValueOrNil, "inputs"); inputs != nil {
+							var children metafileArray
+
+							for _, input := range inputs.Properties {
+								if bytesInOutput := getObjectPropertyNumber(input.ValueOrNil, "bytesInOutput"); bytesInOutput != nil && bytesInOutput.Value > 0 {
+									children = append(children, metafileEntry{
+										name: js_lexer.UTF16ToString(input.Key.Data.(*js_ast.EString).Value),
+										size: int(bytesInOutput.Value),
+									})
+								}
+							}
+
+							sort.Sort(children)
+
+							entries = append(entries, metafileEntry{
+								name:       key,
+								size:       int(bytes.Value),
+								entries:    children,
+								entryPoint: entryPointPath,
+							})
+						}
+					}
+				}
+			}
+
+			sort.Sort(entries)
+
+			type importData struct {
+				imports []string
+			}
+
+			type graphData struct {
+				parent string
+				depth  uint32
+			}
+
+			importsForPath := make(map[string]importData)
+
+			// Scan over the "inputs" object
+			if inputs := getObjectPropertyObject(result, "inputs"); inputs != nil {
+				for _, prop := range inputs.Properties {
+					if imports := getObjectPropertyArray(prop.ValueOrNil, "imports"); imports != nil {
+						var data importData
+
+						for _, item := range imports.Items {
+							if path := getObjectPropertyString(item, "path"); path != nil {
+								data.imports = append(data.imports, js_lexer.UTF16ToString(path.Value))
+							}
+						}
+
+						importsForPath[js_lexer.UTF16ToString(prop.Key.Data.(*js_ast.EString).Value)] = data
+					}
+				}
+			}
+
+			// Returns a graph with links pointing from imports to importers
+			graphForEntryPoints := func(worklist []string) map[string]graphData {
+				if !opts.Verbose {
+					return nil
+				}
+
+				graph := make(map[string]graphData)
+
+				for _, entryPoint := range worklist {
+					graph[entryPoint] = graphData{}
+				}
+
+				for len(worklist) > 0 {
+					top := worklist[len(worklist)-1]
+					worklist = worklist[:len(worklist)-1]
+					childDepth := graph[top].depth + 1
+
+					for _, importPath := range importsForPath[top].imports {
+						imported, ok := graph[importPath]
+						if !ok {
+							imported.depth = math.MaxUint32
+						}
+
+						if imported.depth > childDepth {
+							imported.depth = childDepth
+							imported.parent = top
+							graph[importPath] = imported
+							worklist = append(worklist, importPath)
+						}
+					}
+				}
+
+				return graph
+			}
+
+			graphForAllEntryPoints := graphForEntryPoints(entryPoints)
+
+			type tableEntry struct {
+				first      string
+				second     string
+				third      string
+				firstLen   int
+				secondLen  int
+				thirdLen   int
+				isTopLevel bool
+			}
+
+			var table []tableEntry
+			var colors logger.Colors
+
+			if opts.Color {
+				colors = logger.TerminalColors
+			}
+
+			// Build up the table with an entry for each output file (other than ".map" files)
+			for _, entry := range entries {
+				second := prettyPrintByteCount(entry.size)
+				third := "100.0%"
+
+				table = append(table, tableEntry{
+					first:      fmt.Sprintf("%s%s%s", colors.Bold, entry.name, colors.Reset),
+					firstLen:   utf8.RuneCountInString(entry.name),
+					second:     fmt.Sprintf("%s%s%s", colors.Bold, second, colors.Reset),
+					secondLen:  len(second),
+					third:      fmt.Sprintf("%s%s%s", colors.Bold, third, colors.Reset),
+					thirdLen:   len(third),
+					isTopLevel: true,
+				})
+
+				graph := graphForAllEntryPoints
+				if entry.entryPoint != "" {
+					// If there are multiple entry points and this output file is from an
+					// entry point, prefer import paths for this entry point. This is less
+					// confusing than showing import paths for another entry point.
+					graph = graphForEntryPoints([]string{entry.entryPoint})
+				}
+
+				// Add a sub-entry for each input file in this output file
+				for j, child := range entry.entries {
+					indent := " ├ "
+					if j+1 == len(entry.entries) {
+						indent = " └ "
+					}
+					percent := 100.0 * float64(child.size) / float64(entry.size)
+
+					first := indent + child.name
+					second := prettyPrintByteCount(child.size)
+					third := fmt.Sprintf("%.1f%%", percent)
+
+					table = append(table, tableEntry{
+						first:     first,
+						firstLen:  utf8.RuneCountInString(first),
+						second:    second,
+						secondLen: len(second),
+						third:     third,
+						thirdLen:  len(third),
+					})
+
+					// If we're in verbose mode, also print the import chain from this file
+					// up toward an entry point to show why this file is in the bundle
+					if opts.Verbose {
+						indent = " │ "
+						if j+1 == len(entry.entries) {
+							indent = "   "
+						}
+						data := graph[child.name]
+						depth := 0
+
+						for data.depth != 0 {
+							table = append(table, tableEntry{
+								first: fmt.Sprintf("%s%s%s └ %s%s", indent, colors.Dim, strings.Repeat(" ", depth), data.parent, colors.Reset),
+							})
+							data = graph[data.parent]
+							depth += 3
+						}
+					}
+				}
+			}
+
+			maxFirstLen := 0
+			maxSecondLen := 0
+			maxThirdLen := 0
+
+			// Calculate column widths
+			for _, entry := range table {
+				if maxFirstLen < entry.firstLen {
+					maxFirstLen = entry.firstLen
+				}
+				if maxSecondLen < entry.secondLen {
+					maxSecondLen = entry.secondLen
+				}
+				if maxThirdLen < entry.thirdLen {
+					maxThirdLen = entry.thirdLen
+				}
+			}
+
+			sb := strings.Builder{}
+
+			// Render the columns now that we know the widths
+			for _, entry := range table {
+				prefix := "\n"
+				if !entry.isTopLevel {
+					prefix = ""
+				}
+
+				// Import paths don't have second and third columns
+				if entry.second == "" && entry.third == "" {
+					sb.WriteString(fmt.Sprintf("%s  %s\n",
+						prefix,
+						entry.first,
+					))
+					continue
+				}
+
+				second := entry.second
+				secondTrimmed := strings.TrimRight(second, " ")
+				lineChar := " "
+				extraSpace := 0
+
+				if opts.Verbose {
+					lineChar = "─"
+					extraSpace = 1
+				}
+
+				sb.WriteString(fmt.Sprintf("%s  %s %s%s%s %s %s%s%s %s\n",
+					prefix,
+					entry.first,
+					colors.Dim,
+					strings.Repeat(lineChar, extraSpace+maxFirstLen-entry.firstLen+maxSecondLen-entry.secondLen),
+					colors.Reset,
+					secondTrimmed,
+					colors.Dim,
+					strings.Repeat(lineChar, extraSpace+maxThirdLen-entry.thirdLen+len(second)-len(secondTrimmed)),
+					colors.Reset,
+					entry.third,
+				))
+			}
+
+			return sb.String()
+		}
+	}
+
+	return ""
+}
